//
//  SXVEAudioRecorder.m
//  SXVideoEditDemo
//
//  Created by Yin Xie on 2020/4/27.
//  Copyright © 2020 Yin Xie. All rights reserved.
//

#import "SXVEAudioRecorder.h"


@implementation SXVEAudioRecorder

- (instancetype)initWithPath:(NSString *)filePath {
    if (self = [super init]) {
        _recordUrl = [NSURL fileURLWithPath:filePath];
        [self setupRecorder];
    }
    return self;
}

// 配置录音机
-(void)setupRecorder{
    //设置音频会话
    NSError *sessionError;
    [[AVAudioSession sharedInstance] setCategory:AVAudioSessionCategoryPlayAndRecord error:&sessionError];
    if (sessionError){
        NSLog(@"Error setupRecorder session: %s", sessionError);
    }else{
        [[AVAudioSession sharedInstance] setActive:YES error:&sessionError];
    }
    //录音设置
    //创建录音机
    NSError *error = nil;
    _audioRecorder = [[AVAudioRecorder alloc] initWithURL:_recordUrl settings:_setting error:&error];
    _audioRecorder.delegate = self;
    _audioRecorder.meteringEnabled = YES;//如果要监控声波则必须设置为YES
    [_audioRecorder prepareToRecord];
    if (error) {
         NSLog(@"Error setupRecorder record: %s", error);
    }
}

// 录音设置
-(NSDictionary *)setting {
    if (_setting == nil) {
        NSMutableDictionary *setting = [NSMutableDictionary dictionary];
        //录音格式
        [setting setObject:@(kAudioFormatLinearPCM) forKey:AVFormatIDKey];
        //采样率，8000/11025/22050/44100/96000（影响音频的质量）,8000是电话采样率
        [setting setObject:@(22050) forKey:AVSampleRateKey];
        //通道 , 1/2
        [setting setObject:@(2) forKey:AVNumberOfChannelsKey];
        //采样点位数，分为8、16、24、32, 默认16
        [setting setObject:@(16) forKey:AVLinearPCMBitDepthKey];
        //是否使用浮点数采样
        [setting setObject:@(YES) forKey:AVLinearPCMIsFloatKey];
        // 录音质量
        [setting setObject:@(AVAudioQualityHigh) forKey:AVEncoderAudioQualityKey];
        //....其他设置等
    }
    return _setting;
}


#pragma mark - <************************** 事件 **************************>
// !!!: 开始录制
-(void)startRecord {
    [self setupRecorder];
    if (![self.audioRecorder isRecording]) {
        [self.audioRecorder record];//首次使用应用时如果调用record方法会询问用户是否允许使用麦克风
        //        [self.audioRecorder recordForDuration:60];    // 录音时长
    }
}
// !!!: 暂停录制
-(void)pauseRecord {
    if ([self.audioRecorder isRecording]) {
        [self.audioRecorder pause];
    }
}
// !!!: 恢复录制
-(void)resumeRecord {
    if (![self.audioRecorder isRecording]) {
        [self.audioRecorder record];
    }
}
// !!!: 停止录制
-(void)stopRecord {
    [self.audioRecorder stop];
}

- (void)audioRecorderEncodeErrorDidOccur:(AVAudioRecorder *)recorder error:(NSError *)error {
    NSLog(@"%@", error);
}


@end
