//
//  SXVECutViewController.m
//  SXVideoEditDemo
//
//  Created by Yin Xie on 2020/3/19.
//  Copyright © 2020 Yin Xie. All rights reserved.
//
#import <AVFoundation/AVFoundation.h>
#import "SXVECutViewController.h"
#import "SXVEMenuScrollView.h"
#import "SXUICalculateUtil.h"
const static int kStartMenu = 100;
@interface SXVECutViewController ()<
SXVEMenuDelegate,
UIGestureRecognizerDelegate>
@property (weak, nonatomic) IBOutlet UIView *menuContentView;
@property (weak, nonatomic) IBOutlet UIView *cutContentView;
@property (weak, nonatomic) IBOutlet UISlider *sliderView;
@property (weak, nonatomic) IBOutlet UILabel *startLabel;
@property (weak, nonatomic) IBOutlet UILabel *endTimeLabel;
@property (weak, nonatomic) IBOutlet UIImageView *degreeImageView;
@property (weak, nonatomic) IBOutlet UILabel *degreeLabel;

@property (nonatomic, strong) CALayer           *playerLayer;
@property (nonatomic, strong) AVPlayer          *player;
@property (nonatomic, strong) UIImageView       *cutView;
@property (nonatomic, strong) UIImageView       *shadowView;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *rotationViewLeft;


@property (nonatomic, assign) BOOL isLoad;

@property (nonatomic, assign) CGFloat ratio;
@property (nonatomic, assign) NSInteger selectIndex;
@property (nonatomic, assign) BOOL isMoveCut;
@property (nonatomic, assign) NSInteger moveCutTag;
@property (nonatomic, assign) CGFloat initScale;
@property (nonatomic, assign) CGFloat scale;
@property (nonatomic, assign) CGFloat rotation;

@property (nonatomic, assign) CGFloat rotationScale;

@property (nonatomic, assign) CGFloat playerLayerScale;
@property (nonatomic, assign) CGSize assetSize;;
@end

@implementation SXVECutViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view from its nib.
    
    [self setViews];
    
    _ratio = 0.0;
    _selectIndex = kStartMenu;
    _rotationScale = 1.0;
    _scale = 1.0;
    [self addGesture];
}

- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
    if (!_isLoad) {
        [self setRatioMenu];
        [self setPlayerView];
        [self setCutView];
        [self updateRotaion:0.0];
        _isLoad = YES;
    }
}

- (void)setViews {
    [_sliderView setThumbImage:[UIImage imageNamed:@"white_black_circle"] forState:UIControlStateNormal];
    
}

- (void)setPlayerView {
    
    AVAssetTrack *videoTrack = [_asset tracksWithMediaType:AVMediaTypeVideo].firstObject;
    
    if (videoTrack) {
        _sliderView.maximumValue = _asset.duration.value * 1.0 / _asset.duration.timescale;
        int seconds = (int)_sliderView.maximumValue;
        _endTimeLabel.text = [NSString stringWithFormat:@"%02i:%02i", seconds / 60, seconds % 60];
        
        _playerLayer = [[AVPlayerLayer alloc] init];
        CGSize playerSize = videoTrack.naturalSize;
        CGAffineTransform t = videoTrack.preferredTransform;
        if((t.a == 0 && t.b == 1.0 && t.c == -1.0 && t.d == 0) ||
           (t.a == 0 && t.b == -1.0 && t.c == 1.0 && t.d == 0)) {
            _assetSize = CGSizeMake(playerSize.height, playerSize.width);
        }else {
            _assetSize = playerSize;
        }
        AVPlayerItem *item = [AVPlayerItem playerItemWithAsset:_asset];
        self.player = [AVPlayer playerWithPlayerItem:item];
        ((AVPlayerLayer *)_playerLayer).player = self.player;
    }else {
        [_sliderView setHidden:YES];
        [_endTimeLabel setHidden:YES];
        [_startLabel setHidden:YES];
        _playerLayer = [CALayer layer];
        UIImage *image = [UIImage imageWithContentsOfFile:_asset.URL.path];
        _playerLayer.contents = (__bridge id _Nullable)(image.CGImage);
        _assetSize = image.size;
        _assetSize = CGSizeMake(_assetSize.width * image.scale, _assetSize.height * image.scale);
        
    }
    CGSize clipSize = _cutContentView.bounds.size;
    CGFloat clipScale = _assetSize.width / clipSize.width;
    clipSize.width *= clipScale;
    clipSize.height *= clipScale;
    _playerLayerScale = clipScale;
    _initScale = 1 / [SXUICalculateUtil getScale:_assetSize maxSize:clipSize];
    _scale = _initScale;
    _playerLayer.bounds = CGRectMake(0, 0, _assetSize.width / clipScale, _assetSize.height/ clipScale);
    _playerLayer.affineTransform = CGAffineTransformScale(CGAffineTransformIdentity, _scale, _scale);
    _playerLayer.position = CGPointMake(_cutContentView.frame.size.width/2, _cutContentView.frame.size.height/2);
    [_cutContentView.layer addSublayer:_playerLayer];
    
}

- (void)addGesture {
    UIPanGestureRecognizer *panGesture = [[UIPanGestureRecognizer alloc] initWithTarget:self action:@selector(handlePan:)];
    [_cutContentView addGestureRecognizer:panGesture];
    panGesture.delegate = self;
    UIRotationGestureRecognizer *rotationGesture = [[UIRotationGestureRecognizer alloc] initWithTarget:self action:@selector(handleRotation:)];
    [_cutContentView addGestureRecognizer:rotationGesture];
    rotationGesture.delegate = self;
    UIPinchGestureRecognizer *pinchGesture = [[UIPinchGestureRecognizer alloc] initWithTarget:self action:@selector(handlePinch:)];
    [_cutContentView addGestureRecognizer:pinchGesture];
    pinchGesture.delegate = self;
}

- (void)setCutView {
    CGRect frame = CGRectZero;
    if (_ratio == 0.0) {
         CGFloat scale = 1 / [SXUICalculateUtil getScale:_playerLayer.bounds.size maxSize:_cutContentView.bounds.size];
        frame = CGRectMake(0.0, 0.0, _playerLayer.bounds.size.width * scale, _playerLayer.bounds.size.height * scale);
    }else {
        CGFloat contentRatio = _cutContentView.bounds.size.width / _cutContentView.bounds.size.height;
        if (_ratio > contentRatio) {
            frame = CGRectMake(0.0, 0.0, _cutContentView.bounds.size.width, _cutContentView.bounds.size.width / _ratio);
        }else {
            frame = CGRectMake(0.0, 0.0, _cutContentView.bounds.size.height * _ratio, _cutContentView.bounds.size.height);
        }
        
    }
    if (!_cutView) {
        _cutView = [[UIImageView alloc] initWithFrame:frame];
        [_cutContentView addSubview:_cutView];
    }
    _cutView.frame = frame;
    _cutView.center = CGPointMake(_cutContentView.bounds.size.width / 2, _cutContentView.bounds.size.height / 2);
    
    if (!_shadowView) {
        _shadowView = [[UIImageView alloc] initWithFrame:CGRectMake(0.0, 0.0, _cutContentView.bounds.size.width, _cutContentView.bounds.size.height)];
        [_cutContentView addSubview:_shadowView];
    }
    UIGraphicsBeginImageContext(_shadowView.bounds.size);
    CGContextRef context = UIGraphicsGetCurrentContext();
    CGContextAddRect(context, _cutView.frame);
    CGContextAddRect(context, _shadowView.frame);
    CGContextSetFillColorWithColor(context, [UIColor colorWithWhite:0 alpha:0.8].CGColor);
    CGContextEOFillPath(context);
    _shadowView.image = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    
    CGFloat rotate = atan2f(_playerLayer.affineTransform.b, _playerLayer.affineTransform.a);
    
    CGFloat fabDegree = fabs(rotate);
    

    CGFloat rotationFactor = CGRectGetHeight(_cutView.frame) / CGRectGetWidth(_cutView.frame);
    _initScale = _cutView.bounds.size.width / _playerLayer.bounds.size.width;
    CGFloat playRatio = _playerLayer.bounds.size.height / _playerLayer.bounds.size.width;
    CGRect rotationFrame = CGRectApplyAffineTransform(_cutView.frame, _playerLayer.affineTransform);
    if (CGRectGetHeight(rotationFrame) / CGRectGetWidth(rotationFrame) > playRatio) {
        rotationFactor = 1 / rotationFactor;
        _initScale = _cutView.bounds.size.height / _playerLayer.bounds.size.height;
    }
    
    _rotationScale = rotationFactor * sin(fabDegree) + cos(fabDegree);
    CGFloat minScale = _rotationScale * _initScale;
    if (_scale < minScale) {
        _playerLayer.affineTransform = CGAffineTransformScale(_playerLayer.affineTransform, minScale / _scale, minScale / _scale);
        _scale = minScale;
    }
    [self updateCenter:CGPointZero];
}

- (void)moveCutView:(UIPanGestureRecognizer *)pan {
    UIView *view = pan.view;
    if (_isMoveCut && view.tag != _moveCutTag) {
        return;
    }
    
    switch (pan.state) {
        case UIGestureRecognizerStateBegan:
        {
            _isMoveCut = YES;
            _moveCutTag = view.tag;
        }
            break;
        case UIGestureRecognizerStateChanged:
        {
            switch (_moveCutTag) {
                case 1000://tl
                {
                    
                }
                    break;
                case 1001://tr
                    
                    break;
                    case 1002://bl
                    
                    break;
                    case 1003://br
                    
                    break;
                default:
                    break;
            }
        }
            break;
        case UIGestureRecognizerStateCancelled:
        case UIGestureRecognizerStateEnded:
            _isMoveCut = NO;
            break;
        default:
            _isMoveCut = NO;
            break;
    }
}

- (void)setRatioMenu {
    NSArray *array = @[@"free", @"9_16", @"3_4", @"1_1", @"4_3", @"16_9"];
    SXVEMenuScrollView *menuView = [[SXVEMenuScrollView alloc] initWithFrame:_menuContentView.bounds];
    menuView.backgroundColor = UIColor.blackColor;
    [_menuContentView addSubview:menuView];
    [menuView setMenuArray:array type:MenuTypeFirst startTag: kStartMenu];
    menuView.menuDelegate = self;
}

#pragma mark - Actions

- (IBAction)cutAction:(id)sender {
    CGSize size = _cutView.bounds.size;
    size.width *= _playerLayerScale;
    size.height *= _playerLayerScale;
    CGAffineTransform transform = _playerLayer.affineTransform;
    transform.tx = (_playerLayer.position.x - _cutView.frame.origin.x) * _playerLayerScale;
    transform.ty = (_playerLayer.position.y - _cutView.frame.origin.y) * _playerLayerScale;
    transform = CGAffineTransformTranslate(transform, -_assetSize.width * 0.5, -_assetSize.height * 0.5);
    [_delegate updateSize:size transfrom:transform];
    
    [self dismissViewControllerAnimated:YES completion:nil];
}

- (IBAction)handleRotationPan:(UIPanGestureRecognizer *)sender {
    CGPoint translate = [sender translationInView:sender.view];
    CGFloat maxConstraint = CGRectGetWidth(_degreeImageView.bounds) - 20;
    CGFloat minConstraint = -20;
    CGFloat constraint = MAX(minConstraint, MIN(maxConstraint, _rotationViewLeft.constant + translate.x));
    
    CGFloat length = CGRectGetWidth(_degreeImageView.bounds) / 2;
    CGFloat degree = ((constraint + 20) - length) / length * 45;
    [self updateRotaion:degree];
    [sender setTranslation:CGPointZero inView:sender.view];
}

- (IBAction)controlPlayer:(id)sender {
    [_player seekToTime:CMTimeMake(_sliderView.value * 1000, 1000)];
}

- (IBAction)resetAction:(id)sender {
    [self updateRotaion:0.0];
    _scale = 0.01;
    _playerLayer.affineTransform = CGAffineTransformScale(CGAffineTransformIdentity, _scale, _scale);
    _ratio = 0.0;
    [self setCutView];
    
}
#pragma mark - Gesture
- (void)handlePan:(UIPanGestureRecognizer *)pan {
    if (pan.state == UIGestureRecognizerStateBegan || pan.state == UIGestureRecognizerStateChanged) {
        CGPoint translation = [pan translationInView:_cutContentView];
        [self updateCenter:translation];
        [pan setTranslation:CGPointZero inView:_cutContentView];
    }
}

- (void)handlePinch:(UIPinchGestureRecognizer *)pinch {
    if (pinch.state == UIGestureRecognizerStateBegan || pinch.state == UIGestureRecognizerStateChanged) {
        CGFloat tempScale = pinch.scale;
        if (tempScale * _scale < _rotationScale * _initScale) {
            tempScale = _rotationScale * _initScale / _scale;
        }
        _playerLayer.affineTransform = CGAffineTransformScale(_playerLayer.affineTransform, tempScale, tempScale);
        _scale *= tempScale;
        pinch.scale = 1.0;
    }else {
        [self updateCenter:CGPointZero];
    }
}

- (void)handleRotation:(UIRotationGestureRecognizer *)rotation {
    if (rotation.state == UIGestureRecognizerStateBegan || rotation.state == UIGestureRecognizerStateChanged) {
        CGFloat rotate = atan2f(_playerLayer.affineTransform.b, _playerLayer.affineTransform.a);
        CGFloat degree = rotate / M_PI * 180;
        CGFloat newRationDegree = rotation.rotation / M_PI * 180;
        
        CGFloat currentDegree = degree + newRationDegree;
        
        CGFloat transDegree = MAX(-45, MIN(45, currentDegree));
        
        [self updateRotaion:transDegree];
        [rotation setRotation:0];
    }else {
        [self updateCenter:CGPointZero];
    }
}

- (void)updateCenter:(CGPoint)translation {
    CGFloat rotate = atan2f(_playerLayer.affineTransform.b, _playerLayer.affineTransform.a);
    CGAffineTransform transform;
    CGFloat cs = cos(-rotate), sn = sin(-rotate);
    transform.a = cs;
    transform.b = sn;
    transform.c = -sn;
    transform.d = cs;
    
    CGPoint anchor = _cutView.center;
    transform.tx = anchor.x;
    transform.ty = anchor.y;
    
    
    CGAffineTransform playerTransform = CGAffineTransformTranslate(transform, -anchor.x, -anchor.y);
    CGPoint originCenter = CGPointApplyAffineTransform(_playerLayer.position, playerTransform);
    CGRect bounds = CGRectApplyAffineTransform(_playerLayer.bounds,  CGAffineTransformScale(CGAffineTransformIdentity, _scale, _scale));
    CGRect frame = CGRectMake(originCenter.x - bounds.size.width / 2, originCenter.y - bounds.size.height / 2, bounds.size.width, bounds.size.height);

    CGRect cutFrame = CGRectApplyAffineTransform(_cutView.frame, playerTransform);
    CGFloat centerMaxX = CGRectGetMinX(cutFrame) + CGRectGetWidth(frame) / 2;
    CGFloat centerMaxY = CGRectGetMinY(cutFrame) + CGRectGetHeight(frame) / 2;
    CGFloat centerMinX = CGRectGetMaxX(cutFrame) - CGRectGetWidth(frame) / 2;
    CGFloat centerMinY = CGRectGetMaxY(cutFrame) - CGRectGetHeight(frame) / 2;

//    translation = CGPointApplyAffineTransform(translation, playerTransform);
    CGFloat centerX = MAX(centerMinX, MIN(centerMaxX, originCenter.x + translation.x * _scale));
    CGFloat centerY = MAX(centerMinY, MIN(centerMaxY, originCenter.y + translation.y * _scale));
    
    CGAffineTransform applyTransform;
    CGFloat applycs = cos(rotate), applysn = sin(rotate);
    applyTransform.a = applycs;
    applyTransform.b = applysn;
    applyTransform.c = -applysn;
    applyTransform.d = applycs;
    applyTransform.tx = anchor.x;
    applyTransform.ty = anchor.y;
    applyTransform = CGAffineTransformTranslate(applyTransform, -anchor.x, -anchor.y);
    CGPoint center = CGPointApplyAffineTransform(CGPointMake(centerX, centerY), applyTransform);
    _playerLayer.position = center;
}

- (void)updateRotaion:(CGFloat)transDegree {
    CGFloat length = CGRectGetWidth(_degreeImageView.bounds) / 2;
    _rotationViewLeft.constant = transDegree / 45 * length + length - 20;
    [_degreeImageView layoutIfNeeded];
    _degreeLabel.text = [NSString stringWithFormat:@"%i°", (int)transDegree];
    
    CGFloat rotate = atan2f(_playerLayer.affineTransform.b, _playerLayer.affineTransform.a);
    CGFloat degree = rotate / M_PI * 180;
    
    CGFloat transRotation = (transDegree - degree) / 180 * M_PI;
    
    CGFloat fabDegree = fabs(transDegree) / 180 * M_PI;
    CGFloat rotationFactor = CGRectGetHeight(_cutView.frame) / CGRectGetWidth(_cutView.frame);
    CGFloat playRatio = _playerLayer.bounds.size.height / _playerLayer.bounds.size.width;
    CGRect rotationFrame = CGRectApplyAffineTransform(_cutView.frame, _playerLayer.affineTransform);
    
    CGFloat initScale = _cutView.bounds.size.width / _playerLayer.bounds.size.width;
    if (CGRectGetHeight(rotationFrame) / CGRectGetWidth(rotationFrame) > playRatio) {
        rotationFactor = 1 / rotationFactor;
        initScale = _cutView.bounds.size.height / _playerLayer.bounds.size.height;
    }
    
    CGFloat resetInitScale = initScale / _initScale;
    _initScale = initScale;
    CGFloat scale = rotationFactor * sin(fabDegree) + cos(fabDegree);
    
    
    _playerLayer.affineTransform = CGAffineTransformRotate(_playerLayer.affineTransform, transRotation);
    _playerLayer.affineTransform = CGAffineTransformScale(_playerLayer.affineTransform, scale / _rotationScale * resetInitScale, scale / _rotationScale * resetInitScale);
    _scale *= scale / _rotationScale * resetInitScale;
    _rotationScale = scale;
}

#pragma mark - UIGestureDelegate

- (BOOL)gestureRecognizer:(UIGestureRecognizer *)gestureRecognizer shouldRecognizeSimultaneouslyWithGestureRecognizer:(UIGestureRecognizer *)otherGestureRecognizer {
    if ([otherGestureRecognizer isKindOfClass:[UIRotationGestureRecognizer class]] || [gestureRecognizer isKindOfClass:[UIRotationGestureRecognizer class]]) {
        return NO;
    }
    return YES;
}

#pragma mark - SXVEMenuDelegate

- (void)menuChoose:(SXVEMenuScrollView *)scrollView index:(NSInteger)index {
    CGFloat ratio = 0.0;
    switch (index) {
        case kStartMenu://free
        {
            ratio = 0.0;
        }
            break;
        case kStartMenu + 1://9:16
        {
            ratio = 9.0 / 16;
        }
            break;
        case kStartMenu + 2://3:4
        {
            ratio = 3.0 / 4;
        }
            break;
        case kStartMenu + 3://1:1
        {
            ratio = 1.0;
        }
            break;
        case kStartMenu + 4://4:3
        {
            ratio = 4.0 / 3;
        }
            break;
        case kStartMenu + 5:// 16:9
        {
            ratio = 16.0 / 9;
        }
            break;
        default:
            break;
    }
    if (_ratio == ratio) {
        return;
    }
    _ratio = ratio;
    [self setCutView];
}

- (void)backAction:(SXVEMenuScrollView *)scrollView {
    
}
@end
