//
//  SXVETextViewController.m
//  SXVideoEditDemo
//
//  Created by Yin Xie on 2020/3/30.
//  Copyright © 2020 Yin Xie. All rights reserved.
//

#import "SXVETextViewController.h"
#import "SXVEMenuScrollView.h"
#import "SXVEMenuCollectionViewCell.h"

@interface SXVETextViewController ()<SXVEMenuDelegate, UICollectionViewDelegate, UICollectionViewDataSource>
@property (weak, nonatomic) IBOutlet UITextField *textField;
@property (weak, nonatomic) IBOutlet UIView *styleView;
@property (weak, nonatomic) IBOutlet UICollectionView *bubbleCollectionView;
@property (weak, nonatomic) IBOutlet UIScrollView *animationScrollView;
@property (weak, nonatomic) IBOutlet UIView *textView;
@property (weak, nonatomic) IBOutlet UIView *textColorContentView;
@property (weak, nonatomic) IBOutlet UISlider *textAlphaSliderView;
@property (weak, nonatomic) IBOutlet UIView *strokeView;
@property (weak, nonatomic) IBOutlet UIView *strokeColorContentView;
@property (weak, nonatomic) IBOutlet UIView *wordGapView;
@property (weak, nonatomic) IBOutlet UILabel *wordGapLabel;
@property (weak, nonatomic) IBOutlet UISlider *wordGapSliderView;
@property (weak, nonatomic) IBOutlet UIView *alignmentView;
@property (weak, nonatomic) IBOutlet UIScrollView *fontScrollView;
@property (weak, nonatomic) IBOutlet UISlider *strokeWidthSliderView;
@property (weak, nonatomic) IBOutlet UIView *preparedStyleView;
@property (weak, nonatomic) IBOutlet UICollectionView *animationCollectionView;
@property (weak, nonatomic) IBOutlet UISlider *timeSliderView;
@property (weak, nonatomic) IBOutlet UILabel *animationTimeLabel;
@property (weak, nonatomic) IBOutlet UIButton *inButton;
@property (weak, nonatomic) IBOutlet UIButton *outButton;
@property (weak, nonatomic) IBOutlet UIButton *loopButton;

@property (nonatomic, strong) SXVEMenuScrollView *alignMenu;
@property (nonatomic, strong) SXTextTrack *textTrack;
@property (nonatomic, copy) NSArray *colorArray;
@property (nonatomic, copy) NSString *stylePath;
@property (nonatomic, copy) NSString *bubblePath;
@property (nonatomic, copy) NSString *animationPath;
@property (nonatomic, strong) NSMutableArray *animationArray;
@property (nonatomic, strong) NSMutableArray *styleArray;
@property (nonatomic, strong) NSMutableArray *bubbleArray;
@property (nonatomic, assign) int animationType;

@end

@implementation SXVETextViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view from its nib.
    _colorArray = @[@"fe4a5e", @"ffb034", @"fff21a", @"25e571", @"5be6fe", @"21c7fe", @"b666ff", @"fc5c9f", @"be8150", @"ffffff", @"c3c3c3", @"000000"];
    [_textColorContentView addSubview:[self initoColorView:100]];
    UIView *view = [self initoColorView:200];
    CGRect frame = view.frame;
    frame.origin.x = 60;
    frame.size.width -= 60;
    view.frame = frame;
    [_strokeView addSubview:view];
    [self initPreparedStyle];
    [self initBubble];
    [self initAnimation];
    if (_textTrack) {
        _textField.text = _textTrack.content;
        _strokeWidthSliderView.value = _textTrack.strokeWidth;
        if (_textTrack.strokeWidth == 0) {
            [self noneStrokeAction:nil];
        }
        _wordGapLabel.text = [NSString stringWithFormat:@"%i", (int)_textTrack.tracking];
        _wordGapSliderView.value = _textTrack.tracking;
        _textAlphaSliderView.value = _textTrack.opacity;
        _timeSliderView.maximumValue = _textTrack.duration / 2;
    }
    
    [_bubbleCollectionView registerNib:[UINib nibWithNibName:@"SXVEMenuCollectionViewCell" bundle:nil] forCellWithReuseIdentifier:@"cell"];
    [_animationCollectionView registerNib:[UINib nibWithNibName:@"SXVEMenuCollectionViewCell" bundle:nil] forCellWithReuseIdentifier:@"cell"];
}

- (NSArray *)manageListPath:(NSString *)folderPath {
    NSString *infoPath = [folderPath stringByAppendingPathComponent:@"info.json"];
    NSData *data = [NSData dataWithContentsOfFile:infoPath];
    NSError *error;
    return [NSJSONSerialization JSONObjectWithData:data options:0 error:&error];
}

- (void)initAlignMenu {
    if (!_alignMenu) {
        NSArray *array = @[@"align_left", @"align_center", @"align_right", @"align_top", @"align_center_v", @"align_bottom"];
        _alignMenu = [[SXVEMenuScrollView alloc] initWithFrame:CGRectMake(0.0, 0.0, self.view.bounds.size.width, 80)];
        _alignMenu.tag = 3000;
        _alignMenu.backgroundColor = [UIColor clearColor];
        [_alignmentView addSubview:_alignMenu];
        [_alignMenu setMenuArray:array type:MenuTypeFirst startTag: 100];
        _alignMenu.menuDelegate = self;
    }
}

- (void)initPreparedStyle {
    _stylePath = [[NSBundle mainBundle] pathForResource:@"textStyle" ofType:nil inDirectory:@"test"];
    _styleArray = [NSMutableArray array];
    NSMutableArray *menuArray = [NSMutableArray array];
    [_styleArray addObject:@{@"name" : @"none"}];
    [menuArray addObject:@{@"title" : @"none", @"icon" : @"none"}];
    for (NSDictionary *dic in [self manageListPath:_stylePath]) {
        [_styleArray addObject:dic];
        [menuArray addObject:@{@"title" : dic[@"name"], @"imagePath" : [[_stylePath stringByAppendingPathComponent:dic[@"folder"]] stringByAppendingPathComponent:@"icon.png"]}];
    }
    SXVEMenuScrollView *menu = [[SXVEMenuScrollView alloc] initWithFrame:CGRectMake(0.0, 0.0, self.view.bounds.size.width, 70)];
    menu.tag = 4000;
    menu.backgroundColor = [UIColor clearColor];
    [menu addSubview:_alignMenu];
    [menu setMenuArray:menuArray type:MenuTypeFirst startTag: 200];
    menu.menuDelegate = self;
    [_preparedStyleView addSubview:menu];
    
}

- (void)initBubble {
    _bubblePath = [[NSBundle mainBundle] pathForResource:@"textBubble" ofType:nil inDirectory:@"test"];
    _bubbleArray = [NSMutableArray array];
    [_bubbleArray addObject:@{@"name" : @"none"}];
    [_bubbleArray addObjectsFromArray:[self manageListPath:_bubblePath]];
    
}

- (void)initAnimation {
    _animationPath = [[NSBundle mainBundle] pathForResource:@"textAnimation" ofType:nil inDirectory:@"test"];
    _animationArray = [NSMutableArray array];
    [_animationArray addObject:@{@"name" : @"none"}];
    [_animationArray addObjectsFromArray:[self manageListPath:_animationPath]];
    
}

- (UIView *)initoColorView:(int)startTag {
    UIScrollView *scrollView = [[UIScrollView alloc] initWithFrame:CGRectMake(20.0, 0.0, self.view.bounds.size.width - 20, 50)];
    CGFloat positionX = 0.0;
    CGFloat width = 24.0;
    CGFloat height = 50.0;
    int tag = startTag;
    for (NSString *string in _colorArray) {
        UIButton *button = [[UIButton alloc] initWithFrame:CGRectMake(positionX, 0, width, height)];
        button.backgroundColor = [UIColor colorWithHexString:string];
        [button addTarget:self action:@selector(selectColor:) forControlEvents:UIControlEventTouchUpInside];
        [scrollView addSubview:button];
        button.tag = tag++;
        positionX += width;
    }
    return scrollView;
}

- (void)selectColor:(UIButton *)button {
    CGColorRef color = button.backgroundColor.CGColor;
    size_t numComponents = CGColorGetNumberOfComponents(color);
    if (numComponents==4)
    {
        const CGFloat *components = CGColorGetComponents(color);
        if (button.tag < 200) {
            [_textTrack setFillColorRed:components[0] green:components[1] blue:components[2]];
        }else {
            [_strokeWidthSliderView setHidden:NO];
            [_textTrack setStrokeColorRed:components[0] green:components[1] blue:components[2]];
        }
    }
    [self updateTextLayout];
}

- (void)setModel:(SXVETrackModel *)model {
    _model = model;
    if (_model.track.type == SXTrackTypeText) {
        _textTrack = (SXTextTrack *)_model.track;
    }else {
        _textTrack = nil;
    }
}

- (void)updateTextLayout {
    [_delegate updateTextModel:_model];
}

- (IBAction)textChanged:(id)sender {
    [_textTrack setContent:_textField.text];
    [self updateTextLayout];
}

- (IBAction)keyboard:(id)sender {
    [_textField becomeFirstResponder];
}

- (IBAction)styleAction:(id)sender {
    [_textField resignFirstResponder];
    [_styleView setHidden:NO];
    [_bubbleCollectionView setHidden:YES];
    [_animationScrollView setHidden:YES];
}

- (IBAction)bubbleAction:(id)sender {
    [_textField resignFirstResponder];
    [_styleView setHidden:YES];
    [_bubbleCollectionView setHidden:NO];
    [_animationScrollView setHidden:YES];
}

- (IBAction)animationAction:(id)sender {
    [_textField resignFirstResponder];
    [_styleView setHidden:YES];
    [_bubbleCollectionView setHidden:YES];
    [_animationScrollView setHidden:NO];
    
    [_timeSliderView setHidden: YES];
    [_animationTimeLabel setHidden:YES];

}

- (IBAction)textAction:(id)sender {
    [_textView setHidden:NO];
    [_strokeView setHidden:YES];
    [_wordGapView setHidden:YES];
    [_alignmentView setHidden:YES];
}

- (IBAction)noneStrokeAction:(id)sender {
    _textTrack.strokeWidth = 0.0;
    [_strokeWidthSliderView setHidden:YES];
}

- (IBAction)strokeAction:(id)sender {
    [_textView setHidden:YES];
    [_strokeView setHidden:NO];
    [_wordGapView setHidden:YES];
    [_alignmentView setHidden:YES];
}

- (IBAction)wordGapAction:(id)sender {
    [_textView setHidden:YES];
    [_strokeView setHidden:YES];
    [_wordGapView setHidden:NO];
    [_alignmentView setHidden:YES];
}

- (IBAction)alignmentAction:(id)sender {
    [_textView setHidden:YES];
    [_strokeView setHidden:YES];
    [_wordGapView setHidden:YES];
    [_alignmentView setHidden:NO];
    
    [self initAlignMenu];
}

- (IBAction)textAlphaChanged:(UISlider *)sender {
    _textTrack.opacity = sender.value;
    [self updateTextLayout];
}

- (IBAction)wordGapChanged:(UISlider *)sender {
    _textTrack.tracking = sender.value;
    _wordGapLabel.text = [NSString stringWithFormat:@"%i", (int)_textTrack.tracking];
    [self updateTextLayout];
}

- (IBAction)strokeWidthChanged:(UISlider *)sender {
    _textTrack.strokeWidth = sender.value;
    [self updateTextLayout];
}


- (IBAction)finishAction:(id)sender {
    [self dismissViewControllerAnimated:YES completion:nil];
}

- (IBAction)inAnimation:(id)sender {
    _animationType = 0;
    [_animationCollectionView reloadData];
    [_timeSliderView setHidden: YES];
    [_animationTimeLabel setHidden:YES];
    [_inButton setEnabled:NO];
    [_outButton setEnabled:YES];
    [_loopButton setEnabled:YES];
}

- (IBAction)outAnimation:(id)sender {
    _animationType = 1;
    [_animationCollectionView reloadData];
    [_timeSliderView setHidden: YES];
    [_animationTimeLabel setHidden:YES];
    [_inButton setEnabled:YES];
    [_outButton setEnabled:NO];
    [_loopButton setEnabled:YES];
}

- (IBAction)loppAnimation:(id)sender {
    _animationType = 2;
    [_animationCollectionView reloadData];
    [_timeSliderView setHidden: YES];
    [_animationTimeLabel setHidden:YES];
    [_inButton setEnabled:YES];
    [_outButton setEnabled:YES];
    [_loopButton setEnabled:NO];
}
- (IBAction)timeSliderChanged:(UISlider *)sender {
    _animationTimeLabel.text = [NSString stringWithFormat:@"%.1fs", sender.value];
    if (_animationType == 0 && _model.inTextAnimationEffect) {
        _model.inTextAnimationEffect.duration = sender.value;
        _model.inTextAnimationEffect.durationOfOneCycle = sender.value;
    }else if (_animationType == 1 && _model.outTextAnimationEffect) {
        _model.outTextAnimationEffect.duration = sender.value;
        _model.outTextAnimationEffect.durationOfOneCycle = sender.value;
    }else if (_animationType == 2 && _model.loopTextAnimationEffect) {
        _model.loopTextAnimationEffect.durationOfOneCycle = sender.value;
    }
}

#pragma mark - SXVEMenuDelegate
- (void)menuChoose:(SXVEMenuScrollView *)scrollView index:(NSInteger)index {
    if (scrollView.tag == 4000) {
        if (index == 200) {
            [_textTrack resetTextStyle];
            
        }else {
            NSString *path = [_stylePath stringByAppendingPathComponent:_styleArray[index - 200][@"folder"]];
            int error;
            [_textTrack setTextStyle:path error:&error];

        }
        [self updateTextLayout];
        return;
    }
    switch (index) {
        case 100:
            _textTrack.alignment = SXTextAlignmentTypeLeft;
            _textTrack.direction = SXTextDirectionTypeHorizontal;
            break;
        case 101:
            _textTrack.alignment = SXTextAlignmentTypeCenter;
            _textTrack.direction = SXTextDirectionTypeHorizontal;
            break;
        case 102:
        _textTrack.alignment = SXTextAlignmentTypeRight;
        _textTrack.direction = SXTextDirectionTypeHorizontal;
        break;
        case 103:
        _textTrack.alignment = SXTextAlignmentTypeLeft;
        _textTrack.direction = SXTextDirectionTypeVertical;
        break;
        case 104:
        _textTrack.alignment = SXTextAlignmentTypeCenter;
        _textTrack.direction = SXTextDirectionTypeVertical;
        break;
        case 105:
        _textTrack.alignment = SXTextAlignmentTypeRight;
        _textTrack.direction = SXTextDirectionTypeVertical;
        break;
        default:
            break;
            
    }
    [self updateTextLayout];
}

#pragma mark - collectionView

- (UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath {
    SXVEMenuCollectionViewCell *cell = [collectionView dequeueReusableCellWithReuseIdentifier:@"cell" forIndexPath:indexPath];
    NSDictionary *dic;
    NSString *imagePath;
    if (collectionView == _bubbleCollectionView) {
        dic = _bubbleArray[indexPath.item];
        if (indexPath.item > 0) {
            imagePath = [[_bubblePath stringByAppendingPathComponent:dic[@"folder"]] stringByAppendingPathComponent:dic[@"image"]];
        }
    }else {
        dic = _animationArray[indexPath.item];
        if (indexPath.item > 0) {
            imagePath = [[_animationPath stringByAppendingPathComponent:dic[@"folder"]] stringByAppendingPathComponent:@"icon.gif"];
        }
    }
    
    [cell setTitle:dic[@"name"] imagePath:imagePath];
    return cell;
}

- (NSInteger)numberOfSectionsInCollectionView:(UICollectionView *)collectionView {
    return 1;
}

- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section {
    if (collectionView == _bubbleCollectionView) {
        return _bubbleArray.count;
    }else {
        return _animationArray.count;
    }
}

- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath {
    if (collectionView == _bubbleCollectionView) {
        if (indexPath.item == 0) {
            [_textTrack removeBubble];
        }else {
            NSString *bubblePath = [_bubblePath stringByAppendingPathComponent:_bubbleArray[indexPath.item][@"folder"]];
            int error;
            [_textTrack setBubble:bubblePath error:&error];
        }
    }else {
        if (indexPath.item == 0) {
            [_timeSliderView setHidden:YES];
            [_animationTimeLabel setHidden:YES];
            if (_animationType == 0 && _model.inTextAnimationEffect) {
                [_textTrack removeTextAnimation:_model.inTextAnimationEffect.getEffectId];
                _model.inTextAnimationEffect = nil;
            }else if (_animationType == 1 && _model.outTextAnimationEffect) {
                [_textTrack removeTextAnimation:_model.outTextAnimationEffect.getEffectId];
                _model.outTextAnimationEffect = nil;
            }else if (_animationType == 2 && _model.loopTextAnimationEffect) {
                [_textTrack removeTextAnimation:_model.loopTextAnimationEffect.getEffectId];
                _model.loopTextAnimationEffect = nil;
            }
        }else {
            NSString *animationPath = [_animationPath stringByAppendingPathComponent:_animationArray[indexPath.item][@"folder"]];
            [_timeSliderView setHidden:NO];
            [_animationTimeLabel setHidden:NO];
            if (_animationType == 0) {
                _timeSliderView.maximumValue = _textTrack.duration / 2;
                _timeSliderView.value = _textTrack.duration / 4;
                if (_model.loopTextAnimationEffect) {
                    [_textTrack removeTextAnimation:_model.loopTextAnimationEffect.getEffectId];
                     _model.loopTextAnimationEffect = nil;
                }
                if (_model.inTextAnimationEffect) {
                    [_model.inTextAnimationEffect setResource:animationPath];
                }else {
                    _model.inTextAnimationEffect = [_textTrack addTextAnimation:animationPath];
                    [_model.inTextAnimationEffect setEffectFollowType: SXEffectTimeFollowTypeFollowStart];
                }
                _model.inTextAnimationEffect.duration = _timeSliderView.value;
                _model.inTextAnimationEffect.durationOfOneCycle = _timeSliderView.value;
            }else if (_animationType == 1) {
                _timeSliderView.maximumValue = _textTrack.duration / 2;
                _timeSliderView.value = _textTrack.duration / 4;
                if (_model.loopTextAnimationEffect) {
                    [_textTrack removeTextAnimation:_model.loopTextAnimationEffect.getEffectId];
                     _model.loopTextAnimationEffect = nil;
                }
                if (_model.outTextAnimationEffect) {
                    [_model.outTextAnimationEffect setResource:animationPath];
                }else {
                    _model.outTextAnimationEffect = [_textTrack addTextAnimation:animationPath];
                    [_model.outTextAnimationEffect setEffectFollowType: SXEffectTimeFollowTypeFollowEnd];
                }
                _model.outTextAnimationEffect.duration = _timeSliderView.value;
                _model.outTextAnimationEffect.durationOfOneCycle = _timeSliderView.value;
            }else if (_animationType == 2) {
                _timeSliderView.maximumValue = _textTrack.duration;
                _timeSliderView.value = _textTrack.duration / 4;
                if (_model.inTextAnimationEffect) {
                    [_textTrack removeTextAnimation:_model.inTextAnimationEffect.getEffectId];
                     _model.inTextAnimationEffect = nil;
                }
                if (_model.outTextAnimationEffect) {
                    [_textTrack removeTextAnimation:_model.outTextAnimationEffect.getEffectId];
                     _model.outTextAnimationEffect = nil;
                }
                _timeSliderView.maximumValue = _textTrack.duration;
                if (_model.loopTextAnimationEffect) {
                    [_model.loopTextAnimationEffect setResource:animationPath];
                }else {
                    _model.loopTextAnimationEffect = [_textTrack addTextAnimation:animationPath];
                }
                _model.loopTextAnimationEffect.durationOfOneCycle = _timeSliderView.value;
                [_model.loopTextAnimationEffect setEffectFollowType: SXEffectTimeFollowTypeFollowWholeProcess];
            }
            _animationTimeLabel.text = [NSString stringWithFormat:@"%.1fs", _timeSliderView.value];
        }
    }
    [self updateTextLayout];
}


@end
