//
//  SXVETracksViewController.m
//  SXVideoEditDemo
//
//  Created by Yin Xie on 2020/3/26.
//  Copyright © 2020 Yin Xie. All rights reserved.
//

#import "SXVETracksViewController.h"

#import "SXVEGroupModel.h"
@interface SXVETracksViewController () <UITableViewDelegate, UITableViewDataSource, SXVETrackViewDelegate>
@property (weak, nonatomic) IBOutlet UITableView *tableView;
@property (nonatomic, strong) UIView *timeView;
@property (nonatomic, strong) UIView *controlTrackView;
@property (nonatomic, strong) SXVETrackView *currentTrackView;
@end

@implementation SXVETracksViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view from its nib.
    [self setupViews];
}

- (void)setupViews {
    [_tableView registerClass:[SXVEMainTrackTableViewCell class] forCellReuseIdentifier:@"main"];
    [_tableView registerClass:[SXVETracksTableViewCell class] forCellReuseIdentifier:@"others"];
}

- (void)updateTimes {
    if (!_timeView) {
        _timeView = [[UIView alloc] initWithFrame:CGRectMake(0.0, 0.0, self.view.bounds.size.width, 30)];
        _timeView.backgroundColor = [UIColor blackColor];
    }
    for (UIView *view in _timeView.subviews) {
        [view removeFromSuperview];
    }
    CGFloat startX = [UIScreen mainScreen].bounds.size.width / 2;
    int count = (self.view.bounds.size.width - startX * 2) / _secondLength;
    for (int i = 0; i <= count; i++) {
        UILabel *label = [[UILabel alloc] initWithFrame:CGRectMake(0.0, 0.0, _secondLength, 30.0)];
        label.textAlignment = NSTextAlignmentCenter;
        label.textColor = [UIColor colorWithHexString:@"999999"];
        label.font = [UIFont systemFontOfSize:9];
        label.text = [NSString stringWithFormat:@"%02i:%02i", i / 60, i % 60];
        [_timeView addSubview:label];
        label.center = CGPointMake(startX, _timeView.bounds.size.height / 2);
        startX += _secondLength;
    }
}

- (void)updateTracks:(CGSize)size {
    if (!CGSizeEqualToSize(size, CGSizeZero)) {
        CGRect frame = self.view.frame;
        frame.size.width = size.width;
        self.view.frame = frame;
    }
    [_controlTrackView removeFromSuperview];
    [_tableView reloadData];
}

- (void)updateTrackView {
    [self updateContentSize];
//    [_currentTrackView updateView];
}

- (void)layoutTracks {
    [self updateContentSize];
    int tag = _currentTrackView.tag;
    [_controlTrackView removeFromSuperview];
    [_tableView reloadData];
    SXVETracksTableViewCell *cell = [_tableView cellForRowAtIndexPath:[NSIndexPath indexPathForRow:0 inSection:0]];
    SXVETrackView *trackView = [cell.tracksContentView viewWithTag:tag];
    [self selectTrackView:trackView];
    
}

- (void)initControlTrackView {
    _controlTrackView = [[UIView alloc] init];
    UIImageView *imageView = [[UIImageView alloc] init];
    imageView.image = [[UIImage imageNamed:@"control_border"] resizableImageWithCapInsets:UIEdgeInsetsMake(28, 24, 28, 24) resizingMode:UIImageResizingModeStretch];
    [_controlTrackView addSubview:imageView];
    [imageView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(_controlTrackView).with.mas_offset(7);
        make.right.equalTo(_controlTrackView).with.mas_offset(-7);
        make.top.bottom.equalTo(_controlTrackView);
    }];
    
    UIView *leftView = [[UIView alloc] init];
    [_controlTrackView addSubview:leftView];
    [leftView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.width.mas_equalTo(30);
        make.left.equalTo(_controlTrackView);
        make.top.bottom.equalTo(_controlTrackView);
    }];
    UIPanGestureRecognizer *panLeft = [[UIPanGestureRecognizer alloc] initWithTarget:self action:@selector(controlLeft:)];
    [leftView addGestureRecognizer:panLeft];
    
    UIView *rightView = [[UIView alloc] init];
    [_controlTrackView addSubview:rightView];
    [rightView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.width.mas_equalTo(30);
        make.right.equalTo(_controlTrackView);
        make.top.bottom.equalTo(_controlTrackView);
    }];
    UIPanGestureRecognizer *panRight= [[UIPanGestureRecognizer alloc] initWithTarget:self action:@selector(controlRight:)];
    [rightView addGestureRecognizer:panRight];
}

- (void)updateControlTrackView:(SXVETrackView *)trackView {
    if (!trackView) {
        return;
    }
    if (!_controlTrackView) {
        [self initControlTrackView];
    }
    _currentTrackView = trackView;
    [_controlTrackView removeFromSuperview];
    [trackView.superview addSubview:_controlTrackView];
    [_controlTrackView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(trackView).with.mas_equalTo(-30);
        make.right.equalTo(trackView).with.mas_equalTo(30);
        make.top.equalTo(trackView).with.mas_equalTo(-2);
        make.bottom.equalTo(trackView).with.mas_equalTo(2);
    }];
}

- (void)updateContentSize {
    CGSize size = [_delegate updateContentSize];
    CGRect frame = self.view.frame;
    frame.size.width = size.width;
    self.view.frame = frame;
    [self updateTimes];
    if (_tableViewType == TrackTableViewTypeAudio && _tracksArray.count == 1) {
        [_tableView reloadRowsAtIndexPaths:@[[NSIndexPath indexPathForRow:1 inSection:0]] withRowAnimation:UITableViewRowAnimationNone];
    }
}

- (void)selectFirstTrack {
    SXVETracksTableViewCell *cell = [_tableView cellForRowAtIndexPath:[NSIndexPath indexPathForRow:0 inSection:0]];
    SXVETrackView *trackView = [cell.tracksContentView viewWithTag:500];
    [self selectTrackView:trackView];
}

- (void)selectTrack:(SXVETrackModel *)model {
    int groupIndex = 0;
    int trackIndex = 0;
    BOOL findTrack = NO;
    for (SXVEGroupModel *group in _tracksArray) {
        if (!model.track || [group.trackGroup.getGroupId isEqualToString: model.group.trackGroup.getGroupId]) {
            trackIndex = 0;
            for (SXVETrackModel *trackModel in group.tracks) {
                if (model.track) {
                    if ([trackModel.track.getTrackId isEqualToString:model.track.getTrackId]) {
                        findTrack = YES;
                        break;
                    }
                }else if (model.effect) {
                    if ([trackModel.effect.getEffectId isEqualToString:model.effect.getEffectId]) {
                        findTrack = YES;
                        break;
                        
                    }
                }else if (model.audioItem) {
                    if ([trackModel.audioItem.audioId isEqualToString:model.audioItem.audioId]) {
                        findTrack = YES;
                        break;
                        
                    }
                }
                
                trackIndex++;
            }
            if (findTrack || model.track) {
                break;
            }
        }
        groupIndex++;
    }
    if (groupIndex < _tracksArray.count && trackIndex < model.group.tracks.count) {
        [_tableView scrollToRowAtIndexPath:[NSIndexPath indexPathForRow:groupIndex inSection:0] atScrollPosition:UITableViewScrollPositionMiddle animated:NO];
        SXVETracksTableViewCell *cell = [_tableView cellForRowAtIndexPath:[NSIndexPath indexPathForRow:groupIndex inSection:0]];
        SXVETrackView *trackView = [cell.tracksContentView viewWithTag:500 + trackIndex];
        if (trackView) {
            [self selectTrackView:trackView];
        }
    }
}

- (void)deSelectTrackView {
    _currentTrackView = nil;
    [_controlTrackView removeFromSuperview];
}
#pragma maek - UIGesture

- (void)controlLeft:(UIPanGestureRecognizer *)pan {
    CGPoint translate = [pan translationInView:pan.view];
    [_currentTrackView controlInpointTranslate:translate.x];
    [_currentTrackView.superview layoutIfNeeded];
    [pan setTranslation:CGPointZero inView:pan.view];
    if (pan.state == UIGestureRecognizerStateCancelled || pan.state == UIGestureRecognizerStateEnded) {
        [self updateTrackView];
        if (_currentTrackView.model.type == TrackViewTypeMain) {
            [self layoutTracks];
        }
    }
}

- (void)controlRight:(UIPanGestureRecognizer *)pan {
    CGPoint translate = [pan translationInView:pan.view];
    [_currentTrackView controlEndTranslate:translate.x];
    [_currentTrackView.superview layoutIfNeeded];
    [pan setTranslation:CGPointZero inView:pan.view];
    if (pan.state == UIGestureRecognizerStateCancelled || pan.state == UIGestureRecognizerStateEnded) {
        [self updateContentSize];
        if (_currentTrackView.model.type == TrackViewTypeMain) {
            [self layoutTracks];
        }
    }
}

#pragma mark - SXVETrackViewDelegate
- (void)selectTrackView:(SXVETrackView *)view {
    [_delegate deselectView];
    if (!view.model.track && !view.model.effect && !view.model.audioItem) {
        [_delegate addMusic:[[NSBundle mainBundle].resourcePath stringByAppendingPathComponent:@"test/audio/testmusic.m4a"]];
    }else {
        [self updateControlTrackView:view];
        [_delegate selectTrack:view.model];
    }
}

#pragma mark - tableView

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    if (_tableViewType == TrackTableViewTypeAudio && _tracksArray.count == 1) {
        return 2;
    }
    return _tracksArray.count;
}

- (void)tableView:(UITableView *)tableView willDisplayCell:(UITableViewCell *)cell forRowAtIndexPath:(NSIndexPath *)indexPath {
    cell.contentView.backgroundColor = [UIColor blackColor];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    SXVETracksTableViewCell *cell;
    if (indexPath.row == 0) {
        cell = [tableView dequeueReusableCellWithIdentifier:@"main" forIndexPath:indexPath];
        ((SXVEMainTrackTableViewCell *)cell).delegate = _delegate;
    }else {
        cell = [tableView dequeueReusableCellWithIdentifier:@"others" forIndexPath:indexPath];
        cell.audioCache = _audioCache;
    }
    cell.secondLength = _secondLength;
    if (indexPath.row < _tracksArray.count) {
        SXVEGroupModel *group = _tracksArray[indexPath.row];
        [cell setTracks:group.tracks trackViewDelegate:self];
    }else {
        SXVETrackModel *model = [[SXVETrackModel alloc] init];
        model.type = TrackViewTypeAudio;
//        model.duration = (self.view.bounds.size.width - [UIScreen mainScreen].bounds.size.width) / _secondLength;
        [cell setTracks:@[model] trackViewDelegate:self];
    }
    return cell;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    if (indexPath.row == 0) {
        return 60.0;
    }else {
        return 40.0;
    }
}

- (UIView *)tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section {
    [self updateTimes];
    return _timeView;
}

- (CGFloat)tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section {
    return  30.0;
}




@end
