//
//  SXVEBackgroundView.m
//  SXVideoEditDemo
//
//  Created by Yin Xie on 2020/4/15.
//  Copyright © 2020 Yin Xie. All rights reserved.
//
#import <MobileCoreServices/MobileCoreServices.h>
#import "SXVEBackgroundView.h"
#import "SXVEImageCollectionViewCell.h"

@interface SXVEBackgroundView() <
UIImagePickerControllerDelegate,
UINavigationControllerDelegate>
@property (nonatomic, strong) NSMutableArray *styleArray;
@property (nonatomic, copy) NSArray *colorArray;
@end

@implementation SXVEBackgroundView

- (void)awakeFromNib {
    [super awakeFromNib];
    
    [_styleCollectionView registerNib:[UINib nibWithNibName:@"SXVEImageCollectionViewCell" bundle:nil] forCellWithReuseIdentifier:@"cell"];
}

- (void)setIsSetColor:(BOOL)isSetColor {
    _isSetColor = isSetColor;
    if (_isSetColor) {
        [self initColorView];
    }else {
        [self initStyleArray];
    }
}
 
- (void)initStyleArray {
    _titleLabel.text = L(@"style");
    [_styleView setHidden:NO];
    [_colorView setHidden:YES];
    _styleArray = [NSMutableArray array];
    [_styleArray addObject:@{@"title" : @"none", @"image" : @"none"}];
    [_styleArray addObject:@{@"title" : @"add", @"image": @"add_background"}];
    [_styleCollectionView reloadData];
}

- (void)initColorView {
    _titleLabel.text = L(@"color");
    [_styleView setHidden:YES];
    [_colorView setHidden:NO];
    _colorArray = @[@"fe4a5e", @"ffb034", @"fff21a", @"25e571", @"5be6fe", @"21c7fe", @"b666ff", @"fc5c9f", @"be8150", @"ffffff", @"c3c3c3", @"000000"];
    CGFloat positionX = 0.0;
    CGFloat width = 24.0;
    CGFloat height = 50.0;
    int tag = 100;
    for (NSString *string in _colorArray) {
        UIButton *button = [[UIButton alloc] initWithFrame:CGRectMake(positionX, 0, width, height)];
        button.backgroundColor = [UIColor colorWithHexString:string];
        [button addTarget:self action:@selector(selectColor:) forControlEvents:UIControlEventTouchUpInside];
        [_colorScrollView addSubview:button];
        button.tag = tag++;
        positionX += width;
    }
}

- (void)selectColor:(UIButton *)button {
    CGColorRef color = button.backgroundColor.CGColor;
    size_t numComponents = CGColorGetNumberOfComponents(color);
    if (numComponents==4)
    {
        const CGFloat *components = CGColorGetComponents(color);
        [_editManager setBackgroundColorRed:components[0] green:components[1] blue:components[2]];
        [_delegate  backgroundUpdate];
    }
}

- (IBAction)finishAction:(id)sender {
    [self removeFromSuperview];
}

#pragma mark - UICollectionView

- (NSInteger)numberOfSectionsInCollectionView:(UICollectionView *)collectionView {
    return 1;
}

- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section {
    return  _styleArray.count;
}

- (UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath {
    SXVEImageCollectionViewCell *cell = [collectionView dequeueReusableCellWithReuseIdentifier:@"cell" forIndexPath:indexPath];
    cell.imageView.backgroundColor = [UIColor colorWithHexString:@"242424"];
    cell.imageView.image = [UIImage imageNamed:_styleArray[indexPath.item][@"image"]];
    return cell;
}

- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath {
    if (indexPath.item == 0) {
        [_editManager removeBackground];
        [_delegate backgroundUpdate];
    }else if (indexPath.item == 1) {
        UIImagePickerController *imagePicker = [[UIImagePickerController alloc] init];
        imagePicker.mediaTypes = [[NSArray alloc] initWithObjects:  (NSString *) kUTTypeImage, (NSString *) kUTTypeMovie, nil];
        imagePicker.sourceType = UIImagePickerControllerSourceTypePhotoLibrary;
        imagePicker.delegate = self;
        [_delegate presentViewController:imagePicker];
    }
}

#pragma mark - UIImagePicker
- (void)imagePickerController:(UIImagePickerController *)picker didFinishPickingMediaWithInfo:(NSDictionary<UIImagePickerControllerInfoKey,id> *)info {
    if ([info[UIImagePickerControllerMediaType] isEqualToString:(NSString *) kUTTypeMovie]) {
        NSURL *url = info[UIImagePickerControllerMediaURL];
        NSString *localPath = [[SXFileManager shared] copyFileToProjectFolder:url.path];
        if (localPath) {
            [_editManager setBackground:localPath];
        }
    }else {
        NSURL *url = info[UIImagePickerControllerImageURL];
        NSString *localPath = [[SXFileManager shared] copyFileToProjectFolder:url.path];
        if (localPath) {
            [_editManager setBackground:localPath];
        }
    }
    [_delegate backgroundUpdate];
    [picker dismissViewControllerAnimated:YES completion:nil];
}

- (void)imagePickerControllerDidCancel:(UIImagePickerController *)picker {
    [picker dismissViewControllerAnimated:YES completion:nil];
}

@end
