//
//  SXVECollectionMenuView.m
//  SXVideoEditDemo
//
//  Created by Yin Xie on 2020/3/30.
//  Copyright © 2020 Yin Xie. All rights reserved.
//

#import "SXVECollectionMenuView.h"
#import "SXVEMenuCollectionViewCell.h"

@implementation SXCollectionCellModel



@end


@interface SXVECollectionMenuView()


@property (nonatomic, strong) NSIndexPath *lastSelectIndexPath;

@property (nonatomic, strong) SXAnimationEffect *currentVideoAnimation;
@end

@implementation SXVECollectionMenuView

- (void)awakeFromNib {
    [super awakeFromNib];
    [_sliderView setThumbImage:[UIImage imageNamed:@"red_black_circle"] forState:UIControlStateNormal];
    [_sliderView setMinimumValue:0];
    [_sliderView setMaximumValue:100];
    _startLabel.text = @"0";
    _endLabel.text = @"100";
    [self mas_makeConstraints:^(MASConstraintMaker *make) {
        make.height.mas_equalTo(172.0);
    }];
    
    [_collectionView registerNib:[UINib nibWithNibName:@"SXVEMenuCollectionViewCell" bundle:nil] forCellWithReuseIdentifier:@"cell"];
}

- (void)setIsSingleEffect:(BOOL)isSingleEffect {
    _isSingleEffect = isSingleEffect;
    [_applyAllButton setHidden:!isSingleEffect];
}

- (void)setType:(CollectionType)type {
    _type = type;
    [self setupViews];
}

- (void)setCollectionArray:(NSArray *)collectionArray {
    [_collectionView reloadData];
}

- (void)setupViews {
    switch (_type) {
        case CollectionTypeFilter:
            [self setupFilter];
            break;
        case CollectionTypeAnimation:
            [self setupAnimation];
            break;
        case CollectionTypeAdjustment:
            [self setupAdjustment];
            break;
        case CollectionTypeMask:
            [self setupMask];
        break;
        case CollectionTypeMix:
            [self setBlend];
            break;
        default:
            break;
    }
}

- (NSArray *)manageListPath:(NSString *)folderPath {
    NSString *infoPath = [folderPath stringByAppendingPathComponent:@"info.json"];
    NSData *data = [NSData dataWithContentsOfFile:infoPath];
    NSError *error;
    return [NSJSONSerialization JSONObjectWithData:data options:0 error:&error];
}


- (void)setupFilter {
    [_sliderView setMinimumValue:0];
    [_sliderView setMaximumValue:1];
    _titleLabel.text = L(@"filter");
    [_leftButton setHidden:YES];
    _collectionArray = [NSMutableArray array];
    SXCollectionCellModel *model = [[SXCollectionCellModel alloc] init];
    model.title = @"none";
    model.folderPath = nil;
    [_collectionArray addObject:model];
    NSString *filterPath = [[NSBundle mainBundle] pathForResource:@"filter" ofType:nil inDirectory:@"test"];
    NSArray *array = [self manageListPath:filterPath];
    for (NSDictionary *dic in array) {
        SXCollectionCellModel *model = [[SXCollectionCellModel alloc] init];
        model.title = dic[@"name"];
        model.folderPath = [filterPath stringByAppendingPathComponent:dic[@"folder"]];
        model.imagePath = [model.folderPath stringByAppendingPathComponent:@"icon.png"];
        [_collectionArray addObject:model];
    }
    [_collectionView reloadData];
}

- (void)setupAnimation {
    [_sliderView setMinimumValue:0];
    [_sliderView setMaximumValue:_model.track.duration];
    _startLabel.text = [NSString stringWithFormat:@"%.1fs", 0];
    _endLabel.text = [NSString stringWithFormat:@"%.1fs", _model.track.duration];
    [_sliderView setHidden:YES];
    _titleLabel.text = L(@"animation");
    [_leftButton setHidden:YES];
    _collectionArray = [NSMutableArray array];
    SXCollectionCellModel *model = [[SXCollectionCellModel alloc] init];
    model.title = L(@"none");
    [_collectionArray addObject:model];
    NSString *videoAnimationPath = [[NSBundle mainBundle] pathForResource:@"videoAnimate" ofType:nil inDirectory:@"test"];
    NSArray *array = [self manageListPath:videoAnimationPath];
    for (NSDictionary *dic in array) {
        SXCollectionCellModel *model = [[SXCollectionCellModel alloc] init];
        model.title = dic[@"name"];
        model.folderPath = [videoAnimationPath stringByAppendingPathComponent:dic[@"folder"]];
        model.imagePath = [model.folderPath stringByAppendingPathComponent:@"icon.gif"];
        [_collectionArray addObject:model];
    }
    [_collectionView reloadData];
}

- (void)setupMask {
    [self mas_updateConstraints:^(MASConstraintMaker *make) {
        make.height.mas_equalTo(134);
    }];
    [_collectionView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.height.mas_equalTo(90);
    }];
    
    [_leftButton setTitle:L(@"flip") forState:UIControlStateNormal];
    
    [_sliderContentView setHidden:YES];
    NSArray *array = @[@"none", @"circle", @"retangle", @"polygon", @"star"];
    NSMutableArray *collectionArray = [NSMutableArray array];
    for (NSString *title in array) {
        SXCollectionCellModel *model = [[SXCollectionCellModel alloc] init];
        model.title = title;
        [collectionArray addObject:model];
    }
    _collectionArray = collectionArray;
    [_collectionView reloadData];
    
}

- (void)setupAdjustment {
    
    NSArray *array = @[@"Brightness", @"Contrast", @"Saturation", @"Sharpen", @"Highlight", @"Shadow", @"Exposure", @"Hue"];
    NSMutableArray *collectionArray = [NSMutableArray array];
    for (NSString *title in array) {
        SXCollectionCellModel *model = [[SXCollectionCellModel alloc] init];
        model.title = title;
        [collectionArray addObject:model];
    }
    _collectionArray = collectionArray;
    [_collectionView reloadData];
    [_collectionView selectItemAtIndexPath:[NSIndexPath indexPathForItem:0 inSection:0] animated:NO scrollPosition:UICollectionViewScrollPositionNone];
    [self collectionView:_collectionView didSelectItemAtIndexPath:[NSIndexPath indexPathForItem:0 inSection:0]];
}

- (void)setBlend {
    [_sliderView setHidden:YES];
    NSArray *array = @[@"Normal", @"Lighten", @"Darken", @"Mutiply", @"Average", @"Add", @"Subtract", @"Difference"];
    NSMutableArray *collectionArray = [NSMutableArray array];
    for (NSString *title in array) {
        SXCollectionCellModel *model = [[SXCollectionCellModel alloc] init];
        model.title = title;
        [collectionArray addObject:model];
    }
    _collectionArray = collectionArray;
    [_collectionView reloadData];
    [_collectionView selectItemAtIndexPath:[NSIndexPath indexPathForItem:0 inSection:0] animated:NO scrollPosition:UICollectionViewScrollPositionNone];
    [self collectionView:_collectionView didSelectItemAtIndexPath:[NSIndexPath indexPathForItem:0 inSection:0]];
}

- (void)updateView {
    switch (_type) {
        case CollectionTypeFilter:
            [self updateFilter];
            break;
        case CollectionTypeAnimation:
            [self updateAnimation];
            break;
        case CollectionTypeAdjustment:
            [self updateAdjustment];
            break;
        case CollectionTypeMask:
            [self updateMask];
        break;
        default:
            break;
    }
}

- (void)updateFilter {
    _sliderView.value = _selectValue;
}

- (void)updateAnimation {
    
}

- (void)updateMask {
    
}

- (void)updateAdjustment {
    SXCollectionCellModel *model = _collectionArray[_lastSelectIndexPath.row];
    float value = [_adjustDictionary[model.title] floatValue];
    _sliderView.value = value;
}

- (void)setSelectIndex:(NSInteger)selectIndex {
    _lastSelectIndexPath = [NSIndexPath indexPathForItem:selectIndex inSection:0];
    [_collectionView selectItemAtIndexPath:_lastSelectIndexPath animated:YES scrollPosition:UICollectionViewScrollPositionCenteredHorizontally];
    [self updateView];
}

- (IBAction)finishAction:(id)sender {
    [self removeFromSuperview];
    if (self.type == CollectionTypeMask) {
        [_delegate maskUpdateEnd];
    }
}

- (IBAction)leftButtonAction:(id)sender {
    switch (_type) {
        case CollectionTypeFilter:
        {

        }
            break;
        case CollectionTypeAnimation:
            
            break;
        case CollectionTypeAdjustment:
        {
            SXColorAdjustEffect *colorAdujst;
            if (_isSingleEffect) {
                SXMediaTrack *track = (SXMediaTrack *)_model.track;
                colorAdujst = [track getColorAdjusts].firstObject;
            }else {
                colorAdujst = _colorAdujst;
            }
            if (colorAdujst) {
                [colorAdujst reset];
                _sliderView.value = [colorAdujst getValueForSetting:1<<_lastSelectIndexPath.item];
                [_delegate singleColorAdjustUpdate];
            }
        }
            break;
        case CollectionTypeMask:
        {
            SXMediaTrack *track = (SXMediaTrack *)_model.track;
            [track setInverseMaskShape:![track isInverseMaskShape]];
            [_delegate maskUpdate];
        }
        break;
        default:
            break;
    }
}

- (IBAction)sliderValueChanged:(id)sender {
    switch (_type) {
        case CollectionTypeFilter:
        {
            if (!_currentFilter) {
                SXMediaTrack *track = (SXMediaTrack *)_model.track;
                _currentFilter = track.getFilters.firstObject;
            }
            _currentFilter.alpha = _sliderView.value;
            [_delegate filterUpdate];
        }
            break;
        case CollectionTypeAnimation:
        {
            if (_currentVideoAnimation) {
                _currentVideoAnimation.duration = _sliderView.value;
                _currentVideoAnimation.durationOfOneCycle = _sliderView.value;
            }
            [_delegate animationUpdate];
        }
            break;
        case CollectionTypeAdjustment:
        {
            if (_isSingleEffect) {
                SXMediaTrack *track = (SXMediaTrack *)_model.track;
                SXColorAdjustEffect *colorAdujst = [track getColorAdjusts].firstObject;
                if (colorAdujst) {
                    [colorAdujst setValue:_sliderView.value forSetting:1<<_lastSelectIndexPath.item];
                }
                [_delegate singleColorAdjustUpdate];
            }else {
                if (_colorAdujst) {
                    [_colorAdujst setValue:_sliderView.value forSetting:1<<_lastSelectIndexPath.item];
                }
                [_delegate mainTrackColorAdjustUpdate:_colorAdujst];
            }
            
        }
            break;
        case CollectionTypeMask:
            
        break;
        default:
            break;
    }
}

- (IBAction)applyAll:(id)sender {
    switch (_type) {
        case CollectionTypeFilter:
        {
            for (SXMediaTrack *mediaTrack in _editManager.mainGroup.tracks) {
                for (SXFilterEffect *effect in [mediaTrack getFilters]) {
                    [mediaTrack removeFilter:effect.getEffectId];
                }
                if (_currentFilter) {
                    [mediaTrack addFilterByCopy:_currentFilter];
                }
            }
        }
            break;
        case CollectionTypeAdjustment:
        {
            SXMediaTrack *track = (SXMediaTrack *)_model.track;
            SXColorAdjustEffect *colorAdujst = [track getColorAdjusts].firstObject;
            for (SXMediaTrack *mediaTrack in _editManager.mainGroup.tracks) {
                for (SXColorAdjustEffect *effect in [mediaTrack getColorAdjusts]) {
                    [mediaTrack removeColorAdjust:effect.getEffectId];
                }
                if (colorAdujst) {
                    [mediaTrack addColorAdjustByCopy:colorAdujst];
                }
            }
        }
        default:
            break;
    }
    
}

#pragma mark - collectionView

- (UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath {
    SXVEMenuCollectionViewCell *cell = [collectionView dequeueReusableCellWithReuseIdentifier:@"cell" forIndexPath:indexPath];
    SXCollectionCellModel *model = _collectionArray[indexPath.item];
    [cell setTitle:model.title imagePath:model.imagePath];
    return cell;
}

- (NSInteger)numberOfSectionsInCollectionView:(UICollectionView *)collectionView {
    return 1;
}

- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section {
    return _collectionArray.count;
}

- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath {
    _lastSelectIndexPath = indexPath;
    SXCollectionCellModel *model = _collectionArray[indexPath.item];
    switch (_type) {
        case CollectionTypeFilter:
        {
            if (_isSingleEffect) {
                SXMediaTrack *track = (SXMediaTrack *)_model.track;
                if (!track) {
                    return;
                }
                if (!_currentFilter || !model.folderPath) {
                    for (SXFilterEffect *filter in [track getFilters]) {
                        [track removeFilter:filter.getEffectId];
                    }
                    _currentFilter = nil;
                }
                if (model.folderPath) {
                    if (_currentFilter) {
                        [_currentFilter setResource:model.folderPath];
                    }else {
                        _currentFilter = [track addFilter:model.folderPath];
                        [_currentFilter setEffectFollowType:SXEffectTimeFollowTypeFollowWholeProcess];
                    }
                    _currentFilter.alpha = _sliderView.value;
                }
                [_delegate filterUpdate];
            }else {
                SXMainTrackGroup *mainTrackGroup = (SXMainTrackGroup *)_editManager.mainGroup;
                if (!mainTrackGroup) {
                    return;
                }
                if (!model.folderPath) {
                    if (_currentFilter) {
                        [mainTrackGroup removeFilter:_currentFilter.getEffectId];
                        _currentFilter = nil;
                    }
                }else {
                    if (_currentFilter) {
                        [_currentFilter setResource:model.folderPath];
                    }else {
                        _currentFilter = [mainTrackGroup addFilter:model.folderPath];
                        [_currentFilter setEffectFollowType:SXEffectTimeFollowTypeNone];
                        _currentFilter.startTime = _startTime;
                        _currentFilter.duration = MIN(_editManager.editDuration - _startTime, 3);
                    }
                    _currentFilter.alpha = _sliderView.value;
                }
                [_delegate mainTrackFilterUpdate:_currentFilter];
            }
            
        }
            break;
        case CollectionTypeAnimation:
        {
            SXMediaTrack *track = (SXMediaTrack *)_model.track;
            if (!track) {
                return;
            }
            if (!_currentVideoAnimation || !model.folderPath) {
                [_sliderView setHidden:YES];
                for (SXAnimationEffect *animation in [track getAnimations]) {
                    [track removeAnimation:animation.getEffectId];
                }
                _currentVideoAnimation = nil;
            }
            if (model.folderPath) {
                [_sliderView setHidden:NO];
                if (_currentVideoAnimation) {
                    [_currentVideoAnimation setResource:model.folderPath];
                }else {
                    _currentVideoAnimation = [track addAnimation:model.folderPath];
                    [_currentVideoAnimation setEffectFollowType:SXEffectTimeFollowTypeFollowStart];
                }
                _currentVideoAnimation.duration = _sliderView.value;
                _currentVideoAnimation.durationOfOneCycle = _sliderView.value;
            }
            [_delegate animationUpdate];
        }
            
            break;
        case CollectionTypeAdjustment:
        {
            if (_isSingleEffect) {
                SXMediaTrack *track = (SXMediaTrack *)_model.track;
                SXColorAdjustEffect *colorAdujst = [track getColorAdjusts].firstObject;
                if (!colorAdujst) {
                    colorAdujst = [track addColorAdjust];
                    [colorAdujst setEffectFollowType:SXEffectTimeFollowTypeFollowWholeProcess];
                }
                
                NSArray *range = [SXColorAdjustEffect getValueRangeForColorSetting:1<<indexPath.item];
                float min = [range.firstObject floatValue];
                float max = [range.lastObject floatValue];
                _startLabel.text = [NSString stringWithFormat:@"%.1f", min];
                _endLabel.text = [NSString stringWithFormat:@"%.1f", max];
                _sliderView.minimumValue = min;
                _sliderView.maximumValue = max;
                _sliderView.value = [colorAdujst getValueForSetting:1<<indexPath.item];
            }else {
                SXMainTrackGroup *mainTrackGroup = (SXMainTrackGroup *)_editManager.mainGroup;
                if (!mainTrackGroup) {
                    return;
                }
                if (!_colorAdujst) {
                    _colorAdujst = [mainTrackGroup addColorAdjust];
                    [_colorAdujst setEffectFollowType:SXEffectTimeFollowTypeNone];
                    _colorAdujst.startTime = _startTime;
                    _colorAdujst.duration = MIN(_editManager.editDuration - _startTime, 3);
                }
                NSArray *range = [SXColorAdjustEffect getValueRangeForColorSetting:1<<indexPath.item];
                float min = [range.firstObject floatValue];
                float max = [range.lastObject floatValue];
                _startLabel.text = [NSString stringWithFormat:@"%.1f", min];
                _endLabel.text = [NSString stringWithFormat:@"%.1f", max];
                _sliderView.minimumValue = min;
                _sliderView.maximumValue = max;
                _sliderView.value = [_colorAdujst getValueForSetting:1<<indexPath.item];
                [_delegate mainTrackColorAdjustUpdate:_colorAdujst];
            }
        }
            break;
        case CollectionTypeMask:
        {
            SXMediaTrack *mediaTrack = (SXMediaTrack *)_model.track;
            CGSize trackSize = mediaTrack.getTrackSize;
            CGFloat radius = MIN(trackSize.width, trackSize.height) / 4;
            switch (indexPath.item) {
                case 0:
                {
                    [mediaTrack setMaskShape:nil];
                }
                    break;
                case 1:
                {
                    SXShape *shape = [[SXShape alloc] init];
                    [shape circle:CGPointMake(0.0, 0.0) radius:radius];
                    
                    [shape transform:CGAffineTransformTranslate(CGAffineTransformIdentity, mediaTrack.getTrackSize.width / 2, mediaTrack.getTrackSize.height / 2)];

                    [mediaTrack setMaskShape:@[shape]];
                }
                    break;
               case 2:
                {
                    SXShape *shape = [[SXShape alloc] init];
                    [shape rect:CGRectMake(-radius, -radius, radius * 2, radius * 2)];
                    [shape transform:CGAffineTransformTranslate(CGAffineTransformIdentity, mediaTrack.getTrackSize.width / 2, mediaTrack.getTrackSize.height / 2)];
                    [mediaTrack setMaskShape:@[shape]];
                }
                    break;
                case 3:
                {
                    SXShape *shape = [[SXShape alloc] init];
                    [shape moveTo:CGPointMake(radius, 0)];
                    for (int i = 1; i < 6; i++){
                        [shape lineTo:CGPointMake(radius * cos(360 / 6 * M_PI / 180 * i), radius * sin(360 / 6 * M_PI / 180 * i))];
                    }
                    [shape transform:CGAffineTransformTranslate(CGAffineTransformIdentity, mediaTrack.getTrackSize.width / 2, mediaTrack.getTrackSize.height / 2)];
                    [mediaTrack setMaskShape:@[shape]];
                }
                    break;
                case 4:
                {
                    SXShape *shape = [[SXShape alloc] init];
                    CGFloat R = radius;
                    CGFloat r = radius / 2;
                    for(int i = 0 ; i < 5 ; i ++){
                        if (i == 0) {
                            [shape moveTo:CGPointMake(cos((18+72*i)*M_PI/180) * R,- sin((18+72*i)*M_PI/180) * R)];
                        }else {
                            [shape lineTo:CGPointMake(cos((18+72*i)*M_PI/180) * R,- sin((18+72*i)*M_PI/180) * R)];
                        }
                        [shape lineTo:CGPointMake(cos((54+72*i)*M_PI/180) * r,- sin((54+72*i)*M_PI/180) * r)];
                    }
                    [shape transform:CGAffineTransformTranslate(CGAffineTransformIdentity, mediaTrack.getTrackSize.width / 2, mediaTrack.getTrackSize.height / 2)];
                    [mediaTrack setMaskShape:@[shape]];
                }
                    break;
                default:
                    break;
            }
            
            [_delegate maskUpdate];
        }
        break;
        case CollectionTypeMix:
        {
            SXMediaTrack *mediaTrack = (SXMediaTrack *)_model.track;
            mediaTrack.blendMode = (SXBlendMode)indexPath.item;
            [_delegate blendeModeUpdate];
        }
            break;
        default:
            break;
    }
}

@end
