//
//  SXVEMainTrackTableViewCell.m
//  SXVideoEditDemo
//
//  Created by Yin Xie on 2020/3/26.
//  Copyright © 2020 Yin Xie. All rights reserved.
//


#import "SXVEMainTrackTableViewCell.h"

@interface SXVEMainTrackTableViewCell()
@property (nonatomic, strong) UIButton *audioButton;
@end

@implementation SXVEMainTrackTableViewCell

- (void)awakeFromNib {
    [super awakeFromNib];
    // Initialization code
}

- (void)setSelected:(BOOL)selected animated:(BOOL)animated {
    [super setSelected:selected animated:animated];

    // Configure the view for the selected state
}

- (void)setTracks:(NSArray *)tracks trackViewDelegate:(id<SXVETrackViewDelegate>)delegate {
    [self updateTracksView];
    [self.tracksContentView mas_updateConstraints:^(MASConstraintMaker *make) {
        make.height.mas_equalTo(50.0);
    }];
    if (!_audioButton) {
        _audioButton = [UIButton buttonWithType:UIButtonTypeCustom];
        [self.contentView addSubview:_audioButton];
        [_audioButton setImage:[UIImage imageNamed:@"unmute"] forState:UIControlStateNormal];
        [_audioButton mas_updateConstraints:^(MASConstraintMaker *make) {
            make.right.equalTo(self.tracksContentView.mas_left).with.mas_offset(-10.0);
            make.centerY.equalTo(self.contentView);
            make.width.mas_equalTo(60);
        }];
    }
    UIView *frontView;
    SXVETrackModel *frontModel;
    int buttonTag = 1000;
    int trackViewTag = 500;
    for (SXVETrackModel *model in tracks) {
        SXVETrackView *trackView = [[SXVETrackView alloc] initWithType:model];
        trackView.delegate = delegate;
        [self.tracksContentView addSubview:trackView];
        double overlapTime = 0.0;
        if (frontView && frontModel) {
            overlapTime = frontModel.track.inPoint + frontModel.track.duration - model.track.inPoint;
        }
        trackView.tag = trackViewTag++;
        [trackView mas_updateConstraints:^(MASConstraintMaker *make) {
            if (frontView) {
                make.left.equalTo(frontView.mas_right).with.mas_equalTo(-overlapTime * self.secondLength);
            }else {
                make.left.equalTo(self.tracksContentView).with.mas_equalTo(30.0);
            }
            make.top.bottom.equalTo(self.tracksContentView);
        }];
        if (frontView) {
            UIButton *button = [UIButton buttonWithType:UIButtonTypeCustom];
            [button setImage:[UIImage imageNamed:@"transition"] forState:UIControlStateNormal];
            [button addTarget:self action:@selector(addTransition:) forControlEvents:UIControlEventTouchUpInside];
            button.tag = buttonTag++;
            [self.tracksContentView addSubview:button];
            [button mas_makeConstraints:^(MASConstraintMaker *make) {
                make.centerX.equalTo(frontView.mas_right).with.mas_equalTo(-overlapTime * self.secondLength * 0.5);
                make.centerY.equalTo(frontView.mas_centerY);
                make.width.height.equalTo(frontView.mas_height);
            }];
        }
        frontView = trackView;
        frontModel = model;
    }
    
}

- (void)addTransition:(UIButton *)button {
    [_delegate addTransition:button.tag - 1000];
}


@end
