//
//  SXVERegularSpeedChangeView.m
//  SXVideoEditDemo
//
//  Created by Yin Xie on 2020/3/28.
//  Copyright © 2020 Yin Xie. All rights reserved.
//

#import "SXVERegularSpeedChangeView.h"

@interface SXVERegularSpeedChangeView()
@property (weak, nonatomic) IBOutlet UIImageView *factorImageView;
@property (weak, nonatomic) IBOutlet UIView *panView;
@property (weak, nonatomic) IBOutlet UILabel *factorLabel;

@property (weak, nonatomic) IBOutlet NSLayoutConstraint *panViewLeft;

@end

@implementation SXVERegularSpeedChangeView

- (void)awakeFromNib {
    [super awakeFromNib];
    UIPanGestureRecognizer *pan = [[UIPanGestureRecognizer alloc] initWithTarget:self action:@selector(handlePan:)];
    [_panView addGestureRecognizer:pan];
    
}

- (void)handlePan:(UIPanGestureRecognizer *)sender {
    CGPoint translate = [sender translationInView:sender.view];
    CGFloat maxConstraint = CGRectGetWidth(_factorImageView.bounds) - 20;
    CGFloat minConstraint = -20;
    CGFloat constraint = MAX(minConstraint, MIN(maxConstraint, _panViewLeft.constant + translate.x));
    _panViewLeft.constant = constraint;
    int region = (constraint + 20) / _factorImageView.frame.size.width / 0.2;
    float gap = ((constraint + 20) / _factorImageView.frame.size.width - 0.2 * region) * 5;
    float factor = 1;
    switch (region) {
        case 0:
            factor = gap * 0.9 + 0.1;
            break;
        case 1:
            factor = gap + 1;
            break;
        case 2:
            factor = gap * 3 + 2;
            break;
        case 3:
            factor = gap * 5 + 5;
            break;
        case 4:
            factor = gap * 90 + 10;
            break;
        case 5:
            factor = 100;
            break;
        default:
            break;
    }
    _factorLabel.text = [NSString stringWithFormat:@"%.2f", factor];
    _factor = factor;
    [sender setTranslation:CGPointZero inView:sender.view];
    if (sender.state == UIGestureRecognizerStateCancelled || sender.state == UIGestureRecognizerStateEnded) {
        [_delegate speedChanged:_factor];
    }
}

- (void)setFactor:(CGFloat)factor {
    factor = MIN(100, MAX(0.1, factor));
    CGFloat ratio = 0.2;
    if (factor > 10) {
        ratio = (factor - 10) / 90 / 5 + 0.8;
    }else if (factor > 5) {
        ratio = (factor - 5) / 5 / 5 + 0.6;
    }else if (factor > 2) {
        ratio = (factor - 2) / 3 / 5 + 0.4;
    }else if (factor > 1) {
        ratio = (factor - 1) / 1 / 5 + 0.2;
    }else {
        if (factor < 0.1) {
            factor = 0.1;
        }
        ratio = (factor - 0.1) / 0.9 / 5;
    }
    _panViewLeft.constant = _factorImageView.bounds.size.width * ratio - 20;
    _factorLabel.text = [NSString stringWithFormat:@"%.2f", factor];
    _factor = factor;
}

- (IBAction)finishAction:(id)sender {
    [self removeFromSuperview];
}

@end
