//
//  SXVEStickerView.m
//  SXVideoEditDemo
//
//  Created by Yin Xie on 2020/3/30.
//  Copyright © 2020 Yin Xie. All rights reserved.
//

#import "SXVEStickerView.h"
#import "SXVEImageCollectionViewCell.h"
@interface SXVEStickerView() <UICollectionViewDelegate, UICollectionViewDataSource>
@property (nonatomic, strong) NSMutableArray *stickerArray;
@property (nonatomic, strong) NSString *stickerPath;

@property (nonatomic, strong) NSMutableArray *videoEffectArray;
@property (nonatomic, strong) NSString *videoEffectPath;
@end

@implementation SXVEStickerView

- (instancetype)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
        [self initData];
        [self initViews];
    }
    return self;
}
- (instancetype)init
{
    self = [super init];
    if (self) {
        [self initData];
        [self initViews];
    }
    return self;
}

- (instancetype)initIsVideoEffect:(BOOL)videoEffect {
    self = [super init];
    if (self) {
        self.isVideoEffect = videoEffect;
        [self initData];
        [self initViews];
    }
    return self;
}

- (void)initViews {
    self.backgroundColor = [UIColor blackColor];
    [self mas_makeConstraints:^(MASConstraintMaker *make) {
        make.height.mas_equalTo(300);
    }];
    UIView *menuView = [[UIView alloc] init];
    [self addSubview:menuView];
    [menuView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.left.right.equalTo(self);
        make.height.mas_equalTo(48.0);
    }];
    
    NSMutableArray *cateArray = [NSMutableArray array];
//    if (_isVideoEffect) {
//        [cateArray addObject:@"none"];
//    }
    [cateArray addObject:L(@"自定义")];
    CGFloat positionX = 0.0;
    int tag = 100;
    for (NSString *title in cateArray) {
        UIButton *button = [UIButton buttonWithType:UIButtonTypeCustom];
        [menuView addSubview:button];
        button.frame = CGRectMake(positionX, 0.0, 70, 48.0);
        [button setTitle:title forState:UIControlStateNormal];
        button.tag = tag++;
        [button setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
        button.titleLabel.font = [UIFont systemFontOfSize:12];
        [button addTarget:self action:@selector(changeCategory:) forControlEvents:UIControlEventTouchUpInside];
    }
    
    UIButton *finishButton = [UIButton buttonWithType:UIButtonTypeCustom];
    [finishButton setImage:[UIImage imageNamed:@"finish"] forState:UIControlStateNormal];
    [finishButton addTarget:self action:@selector(finishAction) forControlEvents:UIControlEventTouchUpInside];
    [menuView addSubview:finishButton];
    [finishButton mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.bottom.right.equalTo(menuView);
        make.width.mas_equalTo(60);
    }];
    
    
    UICollectionViewFlowLayout *flowLayout = [[UICollectionViewFlowLayout alloc] init];
    flowLayout.itemSize = CGSizeMake(70, 70);
    flowLayout.minimumInteritemSpacing = 5;
    flowLayout.minimumLineSpacing = 20;
    flowLayout.sectionInset = UIEdgeInsetsMake(20, 20, 20, 20);
    flowLayout.scrollDirection = UICollectionViewScrollDirectionVertical;
    
    UICollectionView *collectionView = [[UICollectionView alloc] initWithFrame:self.frame collectionViewLayout:flowLayout];
    [self addSubview:collectionView];
    [collectionView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(menuView.mas_bottom);
        make.left.right.bottom.equalTo(self);
    }];
    collectionView.delegate = self;
    collectionView.dataSource = self;
    [collectionView registerNib:[UINib nibWithNibName:@"SXVEImageCollectionViewCell" bundle:nil] forCellWithReuseIdentifier:@"cell"];
    
}

- (NSArray *)manageListPath:(NSString *)folderPath {
    NSString *infoPath = [folderPath stringByAppendingPathComponent:@"info.json"];
    NSData *data = [NSData dataWithContentsOfFile:infoPath];
    NSError *error;
    return [NSJSONSerialization JSONObjectWithData:data options:0 error:&error];
}

- (void)initData {
    if (_isVideoEffect) {
        _videoEffectPath = [[NSBundle mainBundle] pathForResource:@"videoEffect" ofType:nil inDirectory:@"test"];
                             
        _videoEffectArray = [[NSMutableArray alloc] initWithArray:[self manageListPath:_videoEffectPath]];
    }else {
        _stickerPath = [[NSBundle mainBundle] pathForResource:@"sticker" ofType:nil inDirectory:@"test"];
                             
        _stickerArray = [[NSMutableArray alloc] initWithArray:[self manageListPath:_stickerPath]];
    }
}

- (void)changeCategory:(UIButton *)button {
    if (_isVideoEffect) {
//        if (button.tag == 100) {
//            SXMainTrackGroup *mainTrackGroup = (SXMainTrackGroup *)_group.trackGroup;
//
//        }
    }
}

- (void)finishAction {
    [self removeFromSuperview];
}

#pragma mark - UICollectionView

- (NSInteger)numberOfSectionsInCollectionView:(UICollectionView *)collectionView {
    return 1;
}

- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section {
    if (_isVideoEffect) {
        return _videoEffectArray.count;
    }
    return  _stickerArray.count;
}

- (UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath {
    SXVEImageCollectionViewCell *cell = [collectionView dequeueReusableCellWithReuseIdentifier:@"cell" forIndexPath:indexPath];
    cell.imageView.backgroundColor = [UIColor colorWithHexString:@"242424"];
    if (_isVideoEffect) {
        NSDictionary *dic = _videoEffectArray[indexPath.item];
        [cell.imageView setGif:[[_videoEffectPath stringByAppendingPathComponent:dic[@"folder"]] stringByAppendingPathComponent:dic[@"image"]]];
    }else {
        NSDictionary *dic = _stickerArray[indexPath.item];
        [cell.imageView setGif:[[_stickerPath stringByAppendingPathComponent:dic[@"folder"]] stringByAppendingPathComponent:@"icon.gif"]];
        
    }
    [cell.imageView setContentMode:UIViewContentModeScaleAspectFit];
    return cell;
}

- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath {
    
    if (_isVideoEffect) {
        NSDictionary *dic = _videoEffectArray[indexPath.item];
        NSString *path = [_videoEffectPath stringByAppendingPathComponent:dic[@"folder"]];
        
        if (!_videoEffect) {
            SXVideoEffect *videoEffect = [_editManager.mainGroup addVideoEffect:path];
            videoEffect.startTime = _startTime;
            videoEffect.duration = MIN(_maxDuration, 3);
            SXVETrackModel *trackModel = [[SXVETrackModel alloc] init];
            trackModel.editManager = _editManager;
            trackModel.type = TrackViewTypeVideoEffect;
            trackModel.effect = videoEffect;
            trackModel.group = _videoEffectGroup;
            [_videoEffectGroup.tracks insertObject:trackModel atIndex:_insertIndex];
            [_delegate addVideoEffectTrack:trackModel];
        }else {
            [_videoEffect setResource:path];
            [_delegate updateVideoEffect];
        }
        [self finishAction];
    }else {
        NSDictionary *dic = _stickerArray[indexPath.item];
        NSString *path = [_stickerPath stringByAppendingPathComponent:dic[@"folder"]];
        SXTrackGroup *group = [_editManager addNewGroup];
        if (group) {
            SXVEGroupModel *groupModel = [[SXVEGroupModel alloc] init];
            groupModel.trackGroup = group;
            SXStickerTrack *track = [_editManager createStickerTrack:path duration:3];
            if (track && [group addTrack:track inPoint:_startTime]) {
                track.transform = CGAffineTransformTranslate(CGAffineTransformIdentity, _editManager.options.width / 2, _editManager.options.height / 2);
                SXVETrackModel *trackModel = [[SXVETrackModel alloc] init];
                trackModel.editManager = _editManager;
                trackModel.type = TrackViewTypeSticker;
                trackModel.track = track;
                trackModel.group = groupModel;
                
                [groupModel.tracks addObject: trackModel];
                [_delegate addGroup:groupModel track:trackModel];
                [self finishAction];
            }
        }
    }
}


@end
