//
//  SXVETrackAnimationView.m
//  SXVideoEditDemo
//
//  Created by Yin Xie on 2020/4/10.
//  Copyright © 2020 Yin Xie. All rights reserved.
//

#import "SXVETrackAnimationView.h"
#import "SXVEMenuCollectionViewCell.h"

@interface SXVETrackAnimationView()
@property (weak, nonatomic) IBOutlet UICollectionView *animationCollectionView;
@property (weak, nonatomic) IBOutlet UISlider *timeSliderView;
@property (weak, nonatomic) IBOutlet UILabel *animationTimeLabel;
@property (weak, nonatomic) IBOutlet UIButton *inButton;
@property (weak, nonatomic) IBOutlet UIButton *outButton;
@property (weak, nonatomic) IBOutlet UIButton *loopButton;

@property (nonatomic, copy) NSString *animationPath;
@property (nonatomic, strong) NSMutableArray *animationArray;
@property (nonatomic, assign) int animationType;

@property (nonatomic, strong) SXStickerTrack *sitckerTrack;
@end

@implementation SXVETrackAnimationView

- (void)awakeFromNib {
    [super awakeFromNib];
    [_animationCollectionView registerNib:[UINib nibWithNibName:@"SXVEMenuCollectionViewCell" bundle:nil] forCellWithReuseIdentifier:@"cell"];
    [self initAnimation];
}

- (void)setModel:(SXVETrackModel *)model {
    _model = model;
    if (_model.track.type == SXTrackTypeSticker) {
        _sitckerTrack = (SXStickerTrack *)_model.track;
    }else {
        _sitckerTrack = nil;
    }
}

- (NSArray *)manageListPath:(NSString *)folderPath {
    NSString *infoPath = [folderPath stringByAppendingPathComponent:@"info.json"];
    NSData *data = [NSData dataWithContentsOfFile:infoPath];
    NSError *error;
    return [NSJSONSerialization JSONObjectWithData:data options:0 error:&error];
}

- (void)initAnimation {
    _animationPath = [[NSBundle mainBundle] pathForResource:@"trackAnimation" ofType:nil inDirectory:@"test"];
    _animationArray = [NSMutableArray array];
    [_animationArray addObject:@{@"name" : @"none"}];
    [_animationArray addObjectsFromArray:[self manageListPath:_animationPath]];
    [_animationCollectionView reloadData];
}

- (IBAction)finishAction:(id)sender {
    [self removeFromSuperview];
}

- (IBAction)inAnimation:(id)sender {
    _animationType = 0;
    [_animationCollectionView reloadData];
    [_timeSliderView setHidden: YES];
    [_animationTimeLabel setHidden:YES];
    [_inButton setEnabled:NO];
    [_outButton setEnabled:YES];
    [_loopButton setEnabled:YES];
}

- (IBAction)outAnimation:(id)sender {
    _animationType = 1;
    [_animationCollectionView reloadData];
    [_timeSliderView setHidden: YES];
    [_animationTimeLabel setHidden:YES];
    [_inButton setEnabled:YES];
    [_outButton setEnabled:NO];
    [_loopButton setEnabled:YES];
}

- (IBAction)loppAnimation:(id)sender {
    _animationType = 2;
    [_animationCollectionView reloadData];
    [_timeSliderView setHidden: YES];
    [_animationTimeLabel setHidden:YES];
    [_inButton setEnabled:YES];
    [_outButton setEnabled:YES];
    [_loopButton setEnabled:NO];
}

- (IBAction)timeSliderChanged:(UISlider *)sender {
    _animationTimeLabel.text = [NSString stringWithFormat:@"%.1fs", sender.value];
    if (_animationType == 0 && _model.inTrackAnimationEffect) {
        _model.inTrackAnimationEffect.duration = sender.value;
        _model.inTrackAnimationEffect.durationOfOneCycle = sender.value;
    }else if (_animationType == 1 && _model.outTrackAnimationEffect) {
        _model.outTrackAnimationEffect.duration = sender.value;
        _model.outTrackAnimationEffect.durationOfOneCycle = sender.value;
    }else if (_animationType == 2 && _model.loopTrackAnimationEffect) {
        _model.loopTrackAnimationEffect.durationOfOneCycle = sender.value;
    }
}
#pragma mark - collectionView

- (UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath {
    SXVEMenuCollectionViewCell *cell = [collectionView dequeueReusableCellWithReuseIdentifier:@"cell" forIndexPath:indexPath];
    NSDictionary *dic;
    dic = _animationArray[indexPath.item];
    NSString *imagePath;
    if (indexPath.item > 0) {
        imagePath = [[_animationPath stringByAppendingPathComponent:dic[@"folder"]] stringByAppendingPathComponent:@"icon.gif"];
    }
    [cell setTitle:dic[@"name"] imagePath:imagePath];
    return cell;
}

- (NSInteger)numberOfSectionsInCollectionView:(UICollectionView *)collectionView {
    return 1;
}

- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section {
    return _animationArray.count;
}

- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath {
if (indexPath.item == 0) {
    [_timeSliderView setHidden:YES];
    [_animationTimeLabel setHidden:YES];
    if (_animationType == 0 && _model.inTrackAnimationEffect) {
        [_sitckerTrack removeTrackAnimation:_model.inTrackAnimationEffect.getEffectId];
        _model.inTrackAnimationEffect = nil;
    }else if (_animationType == 1 && _model.outTrackAnimationEffect) {
        [_sitckerTrack removeTrackAnimation:_model.outTrackAnimationEffect.getEffectId];
        _model.outTrackAnimationEffect = nil;
    }else if (_animationType == 2 && _model.loopTrackAnimationEffect) {
        [_sitckerTrack removeTrackAnimation:_model.loopTrackAnimationEffect.getEffectId];
        _model.loopTrackAnimationEffect = nil;
    }
}else {
    NSString *animationPath = [_animationPath stringByAppendingPathComponent:_animationArray[indexPath.item][@"folder"]];
    [_timeSliderView setHidden:NO];
    [_animationTimeLabel setHidden:NO];
    if (_animationType == 0) {
        _timeSliderView.maximumValue = _sitckerTrack.duration / 2;
        _timeSliderView.value = _sitckerTrack.duration / 4;
        if (_model.loopTrackAnimationEffect) {
            [_sitckerTrack removeTrackAnimation:_model.loopTrackAnimationEffect.getEffectId];
             _model.loopTrackAnimationEffect = nil;
        }
        if (_model.inTrackAnimationEffect) {
            [_model.inTrackAnimationEffect setResource:animationPath];
        }else {
            _model.inTrackAnimationEffect = [_sitckerTrack addTrackAnimation:animationPath];
            [_model.inTrackAnimationEffect setEffectFollowType: SXEffectTimeFollowTypeFollowStart];
        }
        _model.inTrackAnimationEffect.duration = _timeSliderView.value;
        _model.inTrackAnimationEffect.durationOfOneCycle = _timeSliderView.value;
    }else if (_animationType == 1) {
        _timeSliderView.maximumValue = _sitckerTrack.duration / 2;
        _timeSliderView.value = _sitckerTrack.duration / 4;
        if (_model.loopTrackAnimationEffect) {
            [_sitckerTrack removeTrackAnimation:_model.loopTrackAnimationEffect.getEffectId];
             _model.loopTrackAnimationEffect = nil;
        }
        if (_model.outTrackAnimationEffect) {
            [_model.outTrackAnimationEffect setResource:animationPath];
        }else {
            _model.outTrackAnimationEffect = [_sitckerTrack addTrackAnimation:animationPath];
            [_model.outTrackAnimationEffect setEffectFollowType: SXEffectTimeFollowTypeFollowEnd];
        }
        _model.outTrackAnimationEffect.duration = _timeSliderView.value;
        _model.outTrackAnimationEffect.durationOfOneCycle = _timeSliderView.value;
    }else if (_animationType == 2) {
        _timeSliderView.maximumValue = _sitckerTrack.duration;
        _timeSliderView.value = _sitckerTrack.duration / 4;
        if (_model.inTrackAnimationEffect) {
            [_sitckerTrack removeTrackAnimation:_model.inTrackAnimationEffect.getEffectId];
             _model.inTrackAnimationEffect = nil;
        }
        if (_model.outTrackAnimationEffect) {
            [_sitckerTrack removeTrackAnimation:_model.outTrackAnimationEffect.getEffectId];
             _model.outTrackAnimationEffect = nil;
        }
        _timeSliderView.maximumValue = _sitckerTrack.duration;
        if (_model.loopTrackAnimationEffect) {
            [_model.loopTrackAnimationEffect setResource:animationPath];
        }else {
            _model.loopTrackAnimationEffect = [_sitckerTrack addTrackAnimation:animationPath];
        }
        _model.loopTrackAnimationEffect.durationOfOneCycle = _timeSliderView.value;
        [_model.loopTrackAnimationEffect setEffectFollowType: SXEffectTimeFollowTypeFollowWholeProcess];
    }
    _animationTimeLabel.text = [NSString stringWithFormat:@"%.1fs", _timeSliderView.value];
}
}


@end
