//
//  SXVETrackView.m
//  SXVideoEditDemo
//
//  Created by Yin Xie on 2020/3/18.
//  Copyright © 2020 Yin Xie. All rights reserved.
//

#import "SXVETrackView.h"
#import <Masonry/Masonry.h>
#import "SXVEGroupModel.h"
@interface SXVETrackView()

@property (nonatomic, strong) UIView *imageContentView;
@property (nonatomic, strong) SXAudioThumbnail *thumbnail;
@property (nonatomic, assign) double drawTime;
@property (nonatomic, assign) double progress;
@property (nonatomic, assign) BOOL isDraw;
@end


@implementation SXVETrackView

- (instancetype)initWithType:(SXVETrackModel *)model {
    if (self = [super init]) {
        self.clipsToBounds = YES;
        _model = model;
        _secondLength = 54;
        switch (model.type) {
            case TrackViewTypeMain:
                [self setMainTrackView];
                break;
            case TrackViewTypeAudio:
                [self setAudioView];
                break;
            case TrackViewTypeText:
                [self setTextView];
                break;
            case TrackViewTypeSticker:
                [self setStickerView];
            break;
            case TrackViewTypePip:
                [self setPipView];
            break;
            case TrackViewTypeVideoEffect:
                [self setVideoEffect];
            break;
            case TrackViewTypeFilter:
                [self setFilterView];
            break;
            case TrackViewTypeAdjustment:
                [self setAdjustmentView];
            break;
            default:
                break;
        }
        
    }
    [self addGesture];
    return self;
}

- (void)addGesture {
    UITapGestureRecognizer *tap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(handleTap:)];
    [self addGestureRecognizer:tap];
}

- (void)setMainTrackView {
    [self mas_updateConstraints:^(MASConstraintMaker *make) {
        make.width.mas_equalTo(_model.track.duration * _secondLength);
    }];
    _imageContentView = [[UIView alloc] init];
    [self addSubview:_imageContentView];
    [self updateView];
    
}

- (void)setTextView {
    UILabel *titleLabel = [[UILabel alloc] init];
    [self addSubview:titleLabel];
    [titleLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self).with.mas_offset(5);
        make.centerY.equalTo(self);
    }];
    titleLabel.textColor = [UIColor whiteColor];
    titleLabel.font = [UIFont systemFontOfSize:10];
    titleLabel.text = ((SXTextTrack *)_model.track).content;
    self.backgroundColor = [UIColor colorWithHexString:@"fd576a"];
}

- (void)setPipView {
    _imageContentView = [[UIView alloc] init];
    [self addSubview:_imageContentView];
    [self updateView];
}

- (void)setStickerView {
    UILabel *titleLabel = [[UILabel alloc] init];
    [self addSubview:titleLabel];
    [titleLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self).with.mas_offset(5);
        make.centerY.equalTo(self);
    }];
    titleLabel.textColor = [UIColor whiteColor];
    titleLabel.font = [UIFont systemFontOfSize:10];
    titleLabel.text = L(@"sticker");
    self.backgroundColor = [UIColor colorWithHexString:@"8260fa"];
}

- (void)setFilterView {
    UILabel *titleLabel = [[UILabel alloc] init];
    [self addSubview:titleLabel];
    [titleLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self).with.mas_offset(5);
        make.centerY.equalTo(self);
    }];
    titleLabel.textColor = [UIColor whiteColor];
    titleLabel.font = [UIFont systemFontOfSize:10];
    titleLabel.text = L(@"filter");
    self.backgroundColor = [UIColor colorWithHexString:@"8260fa"];
}

- (void)setAdjustmentView {
    UILabel *titleLabel = [[UILabel alloc] init];
    [self addSubview:titleLabel];
    [titleLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self).with.mas_offset(5);
        make.centerY.equalTo(self);
    }];
    titleLabel.textColor = [UIColor whiteColor];
    titleLabel.font = [UIFont systemFontOfSize:10];
    titleLabel.text = L(@"adjust");
    self.backgroundColor = [UIColor colorWithHexString:@"8260fa"];
}

- (void)setVideoEffect {
    UILabel *titleLabel = [[UILabel alloc] init];
    [self addSubview:titleLabel];
    [titleLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self).with.mas_offset(5);
        make.centerY.equalTo(self);
    }];
    titleLabel.textColor = [UIColor whiteColor];
    titleLabel.font = [UIFont systemFontOfSize:10];
    titleLabel.text = L(@"effect");
    self.backgroundColor = [UIColor colorWithHexString:@"8260fa"];
}

- (void)setAudioView {
    UILabel *titleLabel = [[UILabel alloc] init];
    [self addSubview:titleLabel];
    [titleLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self).with.mas_offset(5);
        make.centerY.equalTo(self);
    }];
    titleLabel.textColor = [UIColor whiteColor];
    titleLabel.font = [UIFont systemFontOfSize:10];
    if (_model.audioItem) {
        titleLabel.text = L(@"audio");
        self.backgroundColor = [UIColor colorWithHexString:@"8260fa"];
    }else {
        titleLabel.text = @"+ 添加音频";
        self.backgroundColor = [UIColor colorWithHexString:@"1e1e1e"];
    }
}


- (void)setAudioCache:(SXAudioThumbnailCache *)audioCache {
    _audioCache = audioCache;
    if (_audioCache && _model.audioItem) {
        _thumbnail = [[SXAudioThumbnail  alloc] initWithPath:_model.audioItem.filePath cache:_audioCache singleSamples:512];
        _thumbnail.delegate = self;
        
        _imageContentView = [[UIView alloc] init];
        [self addSubview:_imageContentView];
        [self sendSubviewToBack:_imageContentView];
        [self updateView];
        
    }
}
#pragma mark - Actions

- (void)handleTap:(UITapGestureRecognizer *)tap {
    [_delegate selectTrackView:self];
}

- (void)updateView {
    if (_model.type == TrackViewTypeMain || _model.type == TrackViewTypePip) {
        SXMediaTrack *track = (SXMediaTrack *)_model.track;
        CGFloat width = 50.0;
        CGFloat height = 50.0;
        
        for (UIView *view in _imageContentView.subviews) {
            [view removeFromSuperview];
        }
        if (_model.type == TrackViewTypeMain) {
            [self mas_updateConstraints:^(MASConstraintMaker *make) {
                make.width.mas_equalTo(track.duration * _secondLength);
            }];
        }else {
            height = 34.0;
            CGRect frame = self.frame;
            frame.size.width = track.duration * _secondLength;
            self.frame = frame;
        }
        if (CMTimeGetSeconds(_model.asset.duration) > 0) {
            
            _imageContentView.frame = CGRectMake(-_model.offset * _secondLength, 0.0, _secondLength * track.resourceDuration / [_model speed], height);
            [self analysisAVAsset:_model.asset interval:50 / _secondLength * [_model speed] size:CGSizeMake(width * 2, height * 2) complete:^(int index, UIImage *image, NSError * _Nullable error) {
                dispatch_async(dispatch_get_main_queue(), ^{
                    UIImageView *imageView = [[UIImageView alloc] initWithFrame:CGRectMake(index * width, 0.0, width, height)];
                    imageView.image = image;
                    imageView.contentMode = UIViewContentModeScaleAspectFill;
                    [self->_imageContentView addSubview:imageView];
                });
                
            }];
        }else {
            _imageContentView.frame = CGRectMake(0.0, 0.0, _secondLength * track.duration, height);
            UIImage *image = [UIImage imageWithImage:[UIImage imageWithContentsOfFile:track.getFilePath] size:CGSizeMake(width * 2, height * 2)];
            for (CGFloat positionX = 0.0; positionX < _imageContentView.bounds.size.width + width ; positionX += 50) {
                UIImageView *imageView = [[UIImageView alloc] initWithFrame:CGRectMake(positionX, 0.0, width, height)];
                imageView.image = image;
                [_imageContentView addSubview:imageView];
            }
        }
    }else if (_model.type == TrackViewTypeAudio && _model.audioItem) {
        for (UIView *view in _imageContentView.subviews) {
            [view removeFromSuperview];
        }
        _drawTime = 0.0;
        _imageContentView.frame = CGRectMake(-_model.offset * _secondLength, 0.0, _secondLength * _model.audioItem.resourceDuration / [_model speed], 34);
        if (_thumbnail) {
            [self analysisProgress:_thumbnail.progress];
        }
        
    }
}

#pragma mark - handleTranslate
- (void)controlInpointTranslate:(CGFloat)translate {
    switch (_model.type) {
        case TrackViewTypeMain:
        {
            SXMediaTrack *track = (SXMediaTrack *)_model.track;
            
            NSTimeInterval translateTime = translate / _secondLength;
            NSTimeInterval inPoint = track.inPoint + translateTime;
            inPoint = MAX(track.startTime, MIN(inPoint, track.inPoint + track.duration - 0.1));
            [_model.group.trackGroup setTrack:track.getTrackId inPoint:inPoint];
            

            CGRect frame = _imageContentView.frame;
            frame.origin.x = -_model.offset * _secondLength;
            _imageContentView.frame = frame;
            [self mas_updateConstraints:^(MASConstraintMaker *make) {
                make.width.mas_equalTo(track.duration * _secondLength);
            }];
            if (track.isStaticImage) {
                [self updateView];
            }
        }
            break;
        case TrackViewTypePip: {
            NSTimeInterval translateTime = translate / _secondLength;
            SXMediaTrack *track = (SXMediaTrack *)_model.track;
            SXTrack *lastTrack;
            if (_model.group.tracks.count > 0) {
                for (SXVETrackModel *model in _model.group.tracks) {
                    if ([model.track.getTrackId isEqualToString:track.getTrackId]) {
                        break;
                    }
                    lastTrack = model.track;
                }
            }
            double minTime = 0;
            if (lastTrack) {
                minTime = MAX(lastTrack.inPoint + lastTrack.duration, track.startTime);
            }
            double maxTime = track.inPoint + track.duration - 0.1;
            double inPoint = track.inPoint;
            
            double updateInpointTime = inPoint + translateTime;
            updateInpointTime = MAX(minTime, MIN(maxTime, updateInpointTime));
            updateInpointTime = [_model.editManager removeTimeError:updateInpointTime];
            
            [_model.group.trackGroup setTrack:track.getTrackId inPoint:updateInpointTime];
            
            CGRect frame = self.frame;
            frame.origin.x = _secondLength * track.inPoint + 30;
            frame.size.width = _secondLength * track.duration;
            self.frame = frame;
            
            frame = _imageContentView.frame;
            frame.origin.x = -_model.offset * _secondLength;
            _imageContentView.frame = frame;

        }
            break;
        case TrackViewTypeText:
        case TrackViewTypeSticker:
        {
            SXTrack *track = (SXTrack *)_model.track;
            NSTimeInterval translateTime = translate / _secondLength;
            double minTime = 0.0;
            double maxTime = track.duration + track.inPoint - 0.1;
            translateTime = track.inPoint + translateTime;
            translateTime = MIN(maxTime, MAX(translateTime, minTime));
            translateTime = [_model.editManager removeTimeError:translateTime];
            double duration = track.duration - (translateTime - track.inPoint);
            if ( [_model.group.trackGroup setTrack:track.getTrackId offset:translateTime]) {
                [_model.group.trackGroup setTrack:track.getTrackId duration:duration];
            }
//            NSLog(@"startTime %f", track.startTime);
//            NSLog(@"duration %f", track.duration);
            CGRect frame = self.frame;
            frame.origin.x = _secondLength * track.inPoint + 30;
            frame.size.width = _secondLength * track.duration;
            self.frame = frame;
        }
            break;
        case TrackViewTypeFilter:
        case TrackViewTypeAdjustment:
        case TrackViewTypeVideoEffect:
        {
            NSTimeInterval translateTime = translate / _secondLength;
            SXEffect *effect = _model.effect;
            SXEffect *lastEffect;
            if (_model.group.tracks.count > 0) {
                for (SXVETrackModel *model in _model.group.tracks) {
                    if ([model.effect.getEffectId isEqualToString:effect.getEffectId]) {
                        break;
                    }
                    lastEffect = model.effect;
                }
            }
            double minTime = 0;
            if (lastEffect) {
                minTime = lastEffect.startTime + lastEffect.duration;
            }
            double maxTime = effect.startTime + effect.duration - 0.1;
            double startTime = effect.startTime;
            
            double updateStartTime = startTime + translateTime;
            updateStartTime = MAX(minTime, MIN(maxTime, updateStartTime));
            updateStartTime = [_model.editManager removeTimeError:updateStartTime];
            effect.startTime = updateStartTime;
            effect.duration = effect.duration - (updateStartTime - startTime);
            
            CGRect frame = self.frame;
            frame.origin.x = _secondLength * effect.startTime + 30;
            frame.size.width = _secondLength * effect.duration;
            self.frame = frame;
            
        }
            break;
        case TrackViewTypeAudio:
        {
            SXAudioItem *audioItem = _model.audioItem;
            if (audioItem) {
                NSTimeInterval translateTime = translate / _secondLength;
                SXAudioItem *lastAudioItem;
                if (_model.group.tracks.count > 0) {
                    for (SXVETrackModel *model in _model.group.tracks) {
                        if ([model.audioItem.audioId isEqualToString:audioItem.audioId]) {
                            break;
                        }
                        lastAudioItem = model.audioItem;
                    }
                }
                double minTime = 0;
                if (lastAudioItem) {
                    minTime = MAX(lastAudioItem.inPoint + lastAudioItem.duration, audioItem.startTime);
                }
                double maxTime = audioItem.duration + audioItem.inPoint - 0.1;
                translateTime = audioItem.inPoint + translateTime;
                translateTime = MIN(maxTime, MAX(translateTime, minTime));
                translateTime = [_model.editManager removeTimeError:translateTime];
                audioItem.inPoint = translateTime;
                CGRect frame = self.frame;
                frame.origin.x = _secondLength * audioItem.inPoint + 30;
                frame.size.width = _secondLength * audioItem.duration;
                self.frame = frame;
                
                frame = _imageContentView.frame;
                frame.origin.x = -_model.offset * _secondLength;
                _imageContentView.frame = frame;
            }
            
        }
                break;
        default:
            break;
    }
    
    
    
}

- (void)controlEndTranslate:(CGFloat)translate {
    switch (_model.type) {
        case TrackViewTypeMain:
        {
            SXMediaTrack *track = (SXMediaTrack *)_model.track;
            NSTimeInterval translateTime = translate / _secondLength;
            double duration = track.duration + translateTime;
            if (track.isStaticImage) {
                duration = MAX(0.1, duration);
            }else {
                duration = MAX(0.1, MIN(duration, track.resourceDuration / [_model speed] - _model.offset));
            }
            [_model.group .trackGroup setTrack:track.getTrackId duration:duration];
            
            [self mas_updateConstraints:^(MASConstraintMaker *make) {
                make.width.mas_equalTo(track.duration * _secondLength);
            }];
            if (track.isStaticImage) {
                [self updateView];
            }
        }
            break;
        case TrackViewTypePip:
        {
            NSTimeInterval translateTime = translate / _secondLength;
            SXMediaTrack *track = (SXMediaTrack *)_model.track;
            int index = 0;
            if (_model.group.tracks.count > 0) {
                for (SXVETrackModel *model in _model.group.tracks) {
                    if ([model.track.getTrackId isEqualToString:track.getTrackId]) {
                        break;
                    }
                    index++;
                }
            }
            double durationTime = track.duration;
            
            double updateDurationTime = durationTime + translateTime;
            double minTime = 0.1;
            double maxTime = track.resourceDuration / [_model speed] - _model.offset;
            if (track.isStaticImage) {
                maxTime = MAXFLOAT;
            }
            if (index + 1 < _model.group.tracks.count) {
                SXTrack *nextTrack = ((SXVETrackModel *)_model.group.tracks[index + 1]).track;
                maxTime = MIN(maxTime, nextTrack.inPoint - track.inPoint);
            }
            
            updateDurationTime = MAX(minTime, MIN(maxTime, updateDurationTime));
            
            updateDurationTime = [_model.editManager removeTimeError:updateDurationTime];

            [_model.group.trackGroup setTrack:track.getTrackId duration:updateDurationTime];
            
            CGRect frame = self.frame;
            frame.origin.x = _secondLength * track.inPoint + 30;
            frame.size.width = _secondLength * track.duration;
            self.frame = frame;
        }
            break;
            case TrackViewTypeText:
            case TrackViewTypeSticker:
            {
                SXTrack *track = (SXTrack *)_model.track;
                NSTimeInterval translateTime = translate / _secondLength;
                double duration = track.duration;
                [_model.group.trackGroup setTrack:track.getTrackId duration:duration + translateTime];
                CGRect frame = self.frame;
                frame.origin.x = _secondLength * track.startTime + 30;
                frame.size.width = _secondLength * track.duration;
                self.frame = frame;
            }
                break;
        case TrackViewTypeFilter:
        case TrackViewTypeAdjustment:
         case TrackViewTypeVideoEffect:
               {
                   NSTimeInterval translateTime = translate / _secondLength;
                   SXEffect *effect = _model.effect;
                   int index = 0;
                   if (_model.group.tracks.count > 0) {
                       for (SXVETrackModel *model in _model.group.tracks) {
                           if ([model.effect.getEffectId isEqualToString:effect.getEffectId]) {
                               break;
                           }
                           index++;
                       }
                   }
                   double minTime = 0.1;
                   double maxTime = _model.editManager.editDuration - effect.startTime;
                   if (index + 1 < _model.group.tracks.count) {
                       SXEffect *nextEffect = ((SXVETrackModel *)_model.group.tracks[index + 1]).effect;
                       maxTime = nextEffect.startTime - effect.startTime;
                   }
                   
                   double durationTime = effect.duration;
                   
                   double updateDurationTime = durationTime + translateTime;
                   updateDurationTime = MAX(minTime, MIN(maxTime, updateDurationTime));
                   updateDurationTime = [_model.editManager removeTimeError:updateDurationTime];

                   effect.duration = updateDurationTime;
                   
                   CGRect frame = self.frame;
                   frame.origin.x = _secondLength * effect.startTime + 30;
                   frame.size.width = _secondLength * effect.duration;
                   self.frame = frame;
                   
               }
                   break;
        case TrackViewTypeAudio:
        {
            SXAudioItem *audioItem = _model.audioItem;
            NSTimeInterval translateTime = translate / _secondLength;
            
            int index = 0;
            if (_model.group.tracks.count > 0) {
                for (SXVETrackModel *model in _model.group.tracks) {
                    if ([model.audioItem.audioId isEqualToString:audioItem.audioId]) {
                        break;
                    }
                    index++;
                }
            }
            double durationTime = audioItem.duration;
            
            double updateDurationTime = durationTime + translateTime;
            double minTime = 0.1;
            double maxTime = audioItem.resourceDuration / audioItem.speed - _model.offset;
            if (index + 1 < _model.group.tracks.count) {
                SXAudioItem *nextAudioItem = ((SXVETrackModel *)_model.group.tracks[index + 1]).audioItem;
                maxTime = MIN(maxTime, nextAudioItem.inPoint - audioItem.inPoint);
            }
            
            updateDurationTime = MAX(minTime, MIN(maxTime, updateDurationTime));
            
            updateDurationTime = [_model.editManager removeTimeError:updateDurationTime];

            audioItem.duration = updateDurationTime;
            CGRect frame = self.frame;
            frame.origin.x = _secondLength * audioItem.inPoint + 30;
            frame.size.width = _secondLength * audioItem.duration;
            self.frame = frame;
        }
            break;
        default:
            break;
    }
}
#pragma mark - 编辑视频相关

- (void)analysisAVAsset:(AVAsset *)asset interval:(NSTimeInterval)interval size:(CGSize)size complete:(void (^)(int index, UIImage *image, NSError * _Nullable error))complete
{
    float duration = asset.duration.value * 1.0 / asset.duration.timescale;

    AVAssetImageGenerator *generator = [[AVAssetImageGenerator alloc] initWithAsset:asset];
    generator.maximumSize = size;
    generator.appliesPreferredTrackTransform = YES;
    generator.requestedTimeToleranceBefore = kCMTimeZero;
    generator.requestedTimeToleranceAfter = kCMTimeZero;

    NSMutableArray *arr = [NSMutableArray array];
    for (NSTimeInterval i = 0; i < duration; i += interval) {
        /*
         CMTimeMake(a,b) a当前第几帧, b每秒钟多少帧
         */
        //这里加上0.35 是为了避免解析0s图片必定失败的问题
        CMTime time = CMTimeMake(MAX(0.35, i) * asset.duration.timescale, asset.duration.timescale);
        NSValue *value = [NSValue valueWithCMTime:time];
        [arr addObject:value];
    }

    __block long count = 0;
    [generator generateCGImagesAsynchronouslyForTimes:arr completionHandler:^(CMTime requestedTime, CGImageRef  _Nullable image, CMTime actualTime, AVAssetImageGeneratorResult result, NSError * _Nullable error) {
        switch (result) {
            case AVAssetImageGeneratorSucceeded:
                break;
            case AVAssetImageGeneratorFailed:
                NSLog(@"第%ld秒图片解析失败", count);
                break;
            case AVAssetImageGeneratorCancelled:
                NSLog(@"取消解析视频图片");
                break;
        }
        complete(count, [UIImage imageWithCGImage:image], error);
        count++;
    }];
}


- (void)analysisProgress:(double)progress {
    _progress = progress;
    if (_isDraw) {
        return;
    }
    _isDraw = YES;
    CGFloat loadDuration = progress * _model.audioItem.resourceDuration;
    if (loadDuration > _drawTime) {
        NSArray *thumbArray = [_thumbnail getThumbDataStartTime:_drawTime duration:loadDuration - _drawTime];
        NSArray *firstChannel = thumbArray.firstObject;
        float secondCount = firstChannel.count / (loadDuration - _drawTime) * _model.audioItem.speed;
        if (secondCount > 0) {
            int scale = 1;
            CGFloat strokeWidth = _secondLength / secondCount * scale;
            CGFloat secondLength = _secondLength;
            dispatch_async(dispatch_get_main_queue(), ^{
                __weak typeof(self) weakSelf = self;
                BOOL startContext = YES;
                int drawIndex = 0;
                for (int index = 0; index < firstChannel.count; index++) {
                    if (startContext) {
                        startContext = NO;
                        drawIndex = index;
                        CGFloat width = MIN(secondLength, (loadDuration - weakSelf.drawTime) * secondLength);
                        UIGraphicsBeginImageContext(CGSizeMake(width, weakSelf.bounds.size.height * scale));
                        CGContextRef context = UIGraphicsGetCurrentContext();
                        CGContextSetStrokeColorWithColor(context, UIColor.blackColor.CGColor);
                        CGContextSetLineWidth(context, strokeWidth * 2);
                    }
                    int innerIndex = index - drawIndex;
                    int maxValue = 0;
                    int minValue = 0;
                    for (NSArray *channelValues in thumbArray) {
                        NSArray *value = channelValues[index];
                        if (minValue > [value.firstObject intValue]) {
                            minValue = [value.firstObject intValue];
                        }
                        if (maxValue < [value.lastObject intValue]) {
                            maxValue = [value.lastObject intValue];
                        }
                    }
                    
                    CGPoint startPoint = CGPointMake((innerIndex + 0.5) * strokeWidth, (127 - maxValue) * 1.0 / (127 + 128) * (self.bounds.size.height * scale - 4) + 2);
                    CGPoint endPoint = CGPointMake((innerIndex + 0.5) * strokeWidth, (127 - minValue) * 1.0 / (127 + 128) * (self.bounds.size.height * scale - 4) + 2);
                    CGContextRef context = UIGraphicsGetCurrentContext();
                    CGContextMoveToPoint(context, startPoint.x, startPoint.y);
                    CGContextAddLineToPoint(context, endPoint.x, endPoint.y);
                    CGContextStrokePath(context);
                    BOOL endContext = (int)(index / secondCount) < (int)((index + 1) / secondCount);
                    if (endContext || index == firstChannel.count - 1) {
                        startContext = YES;
                        UIImage *image = UIGraphicsGetImageFromCurrentImageContext();
                        UIGraphicsEndImageContext();
                        UIImageView *imageView = [[UIImageView alloc] init];
                        imageView.image = image;
                        imageView.frame = CGRectMake(weakSelf.drawTime / _model.audioItem.speed * secondLength, 0.0, image.size.width / scale, self.bounds.size.height);
                        [weakSelf.imageContentView addSubview:imageView];
                        weakSelf.drawTime += (imageView.frame.size.width) / secondLength * _model.audioItem
                        .speed;
                    }
                }
                weakSelf.isDraw = NO;
                if (weakSelf.drawTime / weakSelf.model.audioItem.resourceDuration < weakSelf.progress) {
                    [weakSelf analysisProgress:weakSelf.progress];
                }
            });
            
        }
        
    }else {
        _isDraw = NO;
         if (_drawTime / _model.audioItem.resourceDuration < _progress) {
             [self analysisProgress:_progress];
         }
    }
}

@end
