//
//  SXVETransitionView.m
//  SXVideoEditDemo
//
//  Created by Yin Xie on 2020/4/7.
//  Copyright © 2020 Yin Xie. All rights reserved.
//

#import "SXVETransitionView.h"
#import "SXVEMenuCollectionViewCell.h"
@interface SXVETransitionView()
@property (weak, nonatomic) IBOutlet UIView *menuView;
@property (weak, nonatomic) IBOutlet UICollectionView *collectionView;
@property (weak, nonatomic) IBOutlet UIView *timeView;
@property (weak, nonatomic) IBOutlet UILabel *maxDurationLabel;
@property (weak, nonatomic) IBOutlet UILabel *durationLabel;
@property (weak, nonatomic) IBOutlet UISlider *sliderView;

@property (nonatomic, strong) NSMutableArray *transitionArray;
@property (nonatomic, strong) NSString *transitionPath;

@property (nonatomic, assign) int indexSelect;
@end

@implementation SXVETransitionView

- (void)awakeFromNib {
    [super awakeFromNib];
    [self initViews];
    [self initData];
}

- (NSArray *)manageListPath:(NSString *)folderPath {
    NSString *infoPath = [folderPath stringByAppendingPathComponent:@"info.json"];
    NSData *data = [NSData dataWithContentsOfFile:infoPath];
    NSError *error;
    return [NSJSONSerialization JSONObjectWithData:data options:0 error:&error];
}

- (void)initData {
    _transitionPath = [[NSBundle mainBundle] pathForResource:@"transition" ofType:nil inDirectory:@"test"];
    
    _transitionArray = [NSMutableArray array];
    
    [_transitionArray addObject:@{@"name" : @"none"}];
    [_transitionArray addObjectsFromArray:[self manageListPath:_transitionPath]];
}

- (void)setModel:(SXVETrackModel *)model {
    _model = model;
    if (_editManager && _model) {
        
        double maxDuration = [[_editManager mainGroup] getTransitionMaxDuration:_model.track.getTrackId];
        _maxDurationLabel.text = [NSString stringWithFormat:@"%.1fs", maxDuration];
        [_sliderView setMaximumValue:maxDuration];
    }
}

- (void)setEditManager:(SXEditManager *)editManager {
    _editManager = editManager;
    if (_editManager && _model) {
        
        double maxDuration = [[_editManager mainGroup] getTransitionMaxDuration:_model.track.getTrackId];
        _maxDurationLabel.text = [NSString stringWithFormat:@"%.1fs", maxDuration];
        [_sliderView setMaximumValue:maxDuration];
    }
}

- (void)initViews {
    
    [_sliderView setThumbImage:[UIImage imageNamed:@"white_black_circle"] forState:UIControlStateNormal];
    
    [_collectionView registerNib:[UINib nibWithNibName:@"SXVEMenuCollectionViewCell" bundle:nil] forCellWithReuseIdentifier:@"cell"];
    
    NSArray *cateArray = @[L(@"基础转场")];
    CGFloat positionX = 0.0;
    int tag = 100;
    for (NSString *title in cateArray) {
        UIButton *button = [UIButton buttonWithType:UIButtonTypeCustom];
        [_menuView addSubview:button];
        button.frame = CGRectMake(positionX, 0.0, 70, 60.0);
        [button setTitle:title forState:UIControlStateNormal];
        button.tag = tag++;
        [button setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
        button.titleLabel.font = [UIFont systemFontOfSize:12];
        [button addTarget:self action:@selector(changeCategory:) forControlEvents:UIControlEventTouchUpInside];
    }
}

- (void)changeCategory:(UIButton *)button {
    
}

- (IBAction)sliderChanged:(id)sender {
    _durationLabel.text = [NSString stringWithFormat:@"%.1fs", _sliderView.value];
}

- (IBAction)applyAll:(id)sender {
    NSString *transitionPath;
    double duration = _sliderView.value;
    if (_indexSelect > 0) {
        NSDictionary *dic = _transitionArray[_indexSelect];
        transitionPath = [_transitionPath stringByAppendingPathComponent:dic[@"folder"]];
    }
    for (SXMediaTrack *mediaTrack in _editManager.mainGroup.tracks) {
        if (transitionPath) {
            int error;
            [_editManager.mainGroup addTransition:transitionPath trackId:mediaTrack.getTrackId duration:duration error:&error];
        }else {
            [_editManager.mainGroup removeTransition:mediaTrack.getTrackId];
        }
    }
}

- (IBAction)finishAction:(id)sender {
    if (_indexSelect == 0) {
        [[_editManager mainGroup] removeTransition:_model.track.getTrackId];
    }else {
        NSDictionary *dic = _transitionArray[_indexSelect];
        int error;
        [[_editManager mainGroup] addTransition:[_transitionPath stringByAppendingPathComponent:dic[@"folder"]] trackId:_model.track.getTrackId duration:_sliderView.value error:&error];
    }
    [_delegate transitionUpdate];
    [self removeFromSuperview];
}

#pragma mark - UICollectionView

- (NSInteger)numberOfSectionsInCollectionView:(UICollectionView *)collectionView {
    return 1;
}

- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section {
    return  _transitionArray.count;
}

- (UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath {
    SXVEMenuCollectionViewCell *cell = [collectionView dequeueReusableCellWithReuseIdentifier:@"cell" forIndexPath:indexPath];
    NSDictionary *dic = _transitionArray[indexPath.row];
    NSString *imagePath;
    if (indexPath.item > 0) {
        imagePath = [[_transitionPath stringByAppendingPathComponent:dic[@"folder"]] stringByAppendingPathComponent:@"icon.gif"];
    }
    [cell setTitle:dic[@"name"] imagePath:imagePath];
    return cell;
}

- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath {
    _indexSelect = indexPath.item;
}

@end
