//
//  SXVEVoiceChangeView.m
//  SXVideoEditDemo
//
//  Created by Yin Xie on 2020/5/7.
//  Copyright © 2020 Yin Xie. All rights reserved.
//

#import "SXVEVoiceChangeView.h"

@interface SXVEVoiceChangeView()

@property (weak, nonatomic) IBOutlet UIImageView *factorImageView;
@property (weak, nonatomic) IBOutlet UIView *panView;
@property (weak, nonatomic) IBOutlet UILabel *factorLabel;

@property (weak, nonatomic) IBOutlet NSLayoutConstraint *panViewLeft;

@end

@implementation SXVEVoiceChangeView

- (void)awakeFromNib {
    [super awakeFromNib];
    UIPanGestureRecognizer *pan = [[UIPanGestureRecognizer alloc] initWithTarget:self action:@selector(handlePan:)];
    [_panView addGestureRecognizer:pan];
    
}

- (void)handlePan:(UIPanGestureRecognizer *)sender {
    CGPoint translate = [sender translationInView:sender.view];
    CGFloat maxConstraint = CGRectGetWidth(_factorImageView.bounds) - 20;
    CGFloat minConstraint = -20;
    CGFloat constraint = MAX(minConstraint, MIN(maxConstraint, _panViewLeft.constant + translate.x));
    _panViewLeft.constant = constraint;
    int region = (constraint + 20) / _factorImageView.frame.size.width * 4;
    float gap = ((constraint + 20) / _factorImageView.frame.size.width - 0.25 * region) * 4;
    float factor = 1;
    switch (region) {
        case 0:
            factor = gap * 0.8 + 0.2;
            break;
        case 1:
            factor = gap + 1;
            break;
        case 2:
            factor = gap + 2;
            break;
        case 3:
            factor = gap + 3;
            break;
        case 4:
            factor = 4;
            break;
        default:
            break;
    }
    _factorLabel.text = [NSString stringWithFormat:@"%.2f", factor];
    _factor = factor;
    [sender setTranslation:CGPointZero inView:sender.view];
    if (sender.state == UIGestureRecognizerStateCancelled || sender.state == UIGestureRecognizerStateEnded) {
        [_delegate speedChanged:_factor isChangePitch:_pitchButton.selected];
    }
}

- (void)setFactor:(CGFloat)factor {
    CGFloat ratio = 0.2;
    if (factor > 1) {
        ratio = (factor - 1) / 4 + 0.25;
    }else {
        if (factor < 0.2) {
            factor = 0.2;
        }
        ratio = (factor - 0.2) / 0.8 / 4;
    }
    _panViewLeft.constant = _factorImageView.bounds.size.width * ratio - 20;
    _factorLabel.text = [NSString stringWithFormat:@"%.2f", factor];
    _factor = factor;
}

- (IBAction)pitchAction:(id)sender {
    _pitchButton.selected = !_pitchButton.selected;
    [_delegate speedChanged:_factor isChangePitch:_pitchButton.selected];
}

- (IBAction)finishAction:(id)sender {
    [self removeFromSuperview];
}

@end
